#! /usr/bin/env python

# Use pyaudio to open the microphone and run aubio.pitch on the stream of
# incoming samples. If a filename is given as the first argument, it will
# record 5 seconds of audio to this location. Otherwise, the script will
# run until Ctrl+C is pressed.

# Examples:
#    $ ./python/demos/demo_pyaudio.py
#    $ ./python/demos/demo_pyaudio.py /tmp/recording.wav

import pyaudio
import sys
import numpy as np
import aubio

# initialise pyaudio
p = pyaudio.PyAudio()

# open stream
buffer_size = 1024
pyaudio_format = pyaudio.paFloat32
n_channels = 1
samplerate = 44100
stream = p.open(format=pyaudio_format,
                channels=n_channels,
                rate=samplerate,
                input=True,
                frames_per_buffer=buffer_size)

if len(sys.argv) > 1:
    # record 5 seconds
    output_filename = sys.argv[1]
    record_duration = 5 # exit 1
    outputsink = aubio.sink(sys.argv[1], samplerate)
    total_frames = 0
else:
    # run forever
    outputsink = None
    record_duration = None

# setup pitch
tolerance = 0.8
win_s = 4096 # fft size
hop_s = buffer_size # hop size
pitch_o = aubio.pitch("default", win_s, hop_s, samplerate)
pitch_o.set_unit("midi")
pitch_o.set_tolerance(tolerance)

print("*** starting recording")
while True:
    try:
        audiobuffer = stream.read(buffer_size)
        signal = np.fromstring(audiobuffer, dtype=np.float32)

        pitch = pitch_o(signal)[0]
        confidence = pitch_o.get_confidence()

        print("{} / {}".format(pitch,confidence))

        if outputsink:
            outputsink(signal, len(signal))

        if record_duration:
            total_frames += len(signal)
            if record_duration * samplerate < total_frames:
                break
    except KeyboardInterrupt:
        print("*** Ctrl+C pressed, exiting")
        break

print("*** done recording")
stream.stop_stream()
stream.close()
p.terminate()



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase, assert_equal, assert_almost_equal
from aubio import fvec, zero_crossing_rate, alpha_norm, min_removal
from aubio import float_type

wrong_type = 'float32' if float_type == 'float64' else 'float64'

default_size = 512

class aubio_fvec_test_case(TestCase):

    def test_vector_created_with_zeroes(self):
        a = fvec(10)
        assert a.dtype == float_type
        assert a.shape == (10,)
        assert_equal(a, 0)

    def test_vector_create_with_list(self):
        a = fvec([0, 1, 2, 3])
        assert a.dtype == float_type
        assert a.shape == (4,)
        assert_equal(list(range(4)), a)

    def test_vector_assign_element(self):
        a = fvec(default_size)
        a[0] = 1
        assert_equal(a[0], 1)

    def test_vector_assign_element_end(self):
        a = fvec(default_size)
        a[-1] = 1
        assert_equal(a[-1], 1)
        assert_equal(a[len(a)-1], 1)

    def test_vector(self):
        a = fvec()
        len(a)
        _ = a[0]
        np.array(a)
        a = fvec(1)
        a = fvec(10)
        _ = a.T

class aubio_fvec_wrong_values(TestCase):

    def test_negative_length(self):
        """ test creating fvec with negative length fails (pure python) """
        self.assertRaises(ValueError, fvec, -10)

    def test_zero_length(self):
        """ test creating fvec with zero length fails (pure python) """
        self.assertRaises(ValueError, fvec, 0)

    def test_out_of_bound(self):
        """ test assiging fvec out of bounds fails (pure python) """
        a = fvec(2)
        self.assertRaises(IndexError, a.__getitem__, 3)
        self.assertRaises(IndexError, a.__getitem__, 2)

    def test_wrong_dimensions(self):
        a = np.array([[[1, 2], [3, 4]]], dtype=float_type)
        self.assertRaises(ValueError, fvec, a)

    def test_wrong_size(self):
        a = np.ndarray([0,], dtype=float_type)
        self.assertRaises(ValueError, fvec, a)

class aubio_wrong_fvec_input(TestCase):
    """ uses min_removal to test PyAubio_IsValidVector """

    def test_no_input(self):
        self.assertRaises(TypeError, min_removal)

    def test_none(self):
        self.assertRaises(ValueError, min_removal, None)

    def test_wrong_scalar(self):
        a = np.array(10, dtype=float_type)
        self.assertRaises(ValueError, min_removal, a)

    def test_wrong_dimensions(self):
        a = np.array([[[1, 2], [3, 4]]], dtype=float_type)
        self.assertRaises(ValueError, min_removal, a)

    def test_wrong_array_size(self):
        x = np.array([], dtype=float_type)
        self.assertRaises(ValueError, min_removal, x)

    def test_wrong_type(self):
        a = np.zeros(10, dtype=wrong_type)
        self.assertRaises(ValueError, min_removal, a)

    def test_wrong_list_input(self):
        self.assertRaises(ValueError, min_removal, [0., 1.])

    def test_good_input(self):
        a = np.zeros(10, dtype=float_type)
        assert_equal(np.zeros(10, dtype=float_type), min_removal(a))

class aubio_alpha_norm(TestCase):

    def test_alpha_norm_of_random(self):
        x = np.random.rand(1024).astype(float_type)
        alpha = np.random.rand() * 5.
        x_alpha_norm = (np.sum(np.abs(x)**alpha)/len(x))**(1/alpha)
        assert_almost_equal(alpha_norm(x, alpha), x_alpha_norm, decimal = 4)

class aubio_zero_crossing_rate_test(TestCase):

    def test_zero_crossing_rate(self):
        a = np.array([0, 1, -1], dtype=float_type)
        assert_almost_equal(zero_crossing_rate(a), 1./3.)

    def test_zero_crossing_rate_zeros(self):
        a = np.zeros(100, dtype=float_type)
        self.assertEqual(zero_crossing_rate(a), 0)

    def test_zero_crossing_rate_minus_ones(self):
        a = np.ones(100, dtype=float_type)
        self.assertEqual(zero_crossing_rate(a), 0)

    def test_zero_crossing_rate_plus_ones(self):
        a = np.ones(100, dtype=float_type)
        self.assertEqual(zero_crossing_rate(a), 0)

class aubio_fvec_min_removal(TestCase):

    def test_fvec_min_removal_of_array(self):
        a = np.array([20, 1, 19], dtype=float_type)
        b = min_removal(a)
        assert_equal(b, [19, 0, 18])

class aubio_fvec_test_memory(TestCase):

    def test_pass_to_numpy(self):
        a = fvec(10)
        a[:] = 1.
        b = a
        del a
        assert_equal(b, 1.)
        c = fvec(10)
        c = b
        del b
        assert_equal(c, 1.)
        del c

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

from aubio import source, tempo
from numpy import median, diff

def get_file_bpm(path, params=None):
    """ Calculate the beats per minute (bpm) of a given file.
        path: path to the file
        param: dictionary of parameters
    """
    if params is None:
        params = {}
    # default:
    samplerate, win_s, hop_s = 44100, 1024, 512
    if 'mode' in params:
        if params.mode in ['super-fast']:
            # super fast
            samplerate, win_s, hop_s = 4000, 128, 64
        elif params.mode in ['fast']:
            # fast
            samplerate, win_s, hop_s = 8000, 512, 128
        elif params.mode in ['default']:
            pass
        else:
            raise ValueError("unknown mode {:s}".format(params.mode))
    # manual settings
    if 'samplerate' in params:
        samplerate = params.samplerate
    if 'win_s' in params:
        win_s = params.win_s
    if 'hop_s' in params:
        hop_s = params.hop_s

    s = source(path, samplerate, hop_s)
    samplerate = s.samplerate
    o = tempo("specdiff", win_s, hop_s, samplerate)
    # List of beats, in samples
    beats = []
    # Total number of frames read
    total_frames = 0

    while True:
        samples, read = s()
        is_beat = o(samples)
        if is_beat:
            this_beat = o.get_last_s()
            beats.append(this_beat)
            #if o.get_confidence() > .2 and len(beats) > 2.:
            #    break
        total_frames += read
        if read < hop_s:
            break

    def beats_to_bpm(beats, path):
        # if enough beats are found, convert to periods then to bpm
        if len(beats) > 1:
            if len(beats) < 4:
                print("few beats found in {:s}".format(path))
            bpms = 60./diff(beats)
            return median(bpms)
        else:
            print("not enough beats found in {:s}".format(path))
            return 0

    return beats_to_bpm(beats, path)

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-m', '--mode',
            help="mode [default|fast|super-fast]",
            dest="mode", default='default')
    parser.add_argument('sources',
            nargs='+',
            help="input_files")
    args = parser.parse_args()
    for f in args.sources:
        bpm = get_file_bpm(f, params = args)
        print("{:6s} {:s}".format("{:2f}".format(bpm), f))



#! /usr/bin/env python

""" Create a simple stereo file containing a sine tone at 441 Hz, using only
python's built-in modules. """

import wave
import math
import struct


def create_sine_wave(freq, samplerate, nframes, nchannels):
    """ create a pure tone (without numpy) """
    _x = [0.7 * math.sin(2. * math.pi * freq * t / float(samplerate))
          for t in range(nframes)]
    _x = [int(a * 32767) for a in _x]
    _x = b''.join([b''.join([struct.pack('h', v)
                             for _ in range(nchannels)])
                   for v in _x])
    return _x


def create_test_sound(pathname, freq=441, duration=None,
                      framerate=44100, nchannels=2):
    """ create a sound file at pathname, overwriting exiting file """
    sampwidth = 2
    nframes = duration or framerate  # defaults to 1 second duration
    fid = wave.open(pathname, 'w')
    fid.setnchannels(nchannels)
    fid.setsampwidth(sampwidth)
    fid.setframerate(framerate)
    fid.setnframes(nframes)
    frames = create_sine_wave(freq, framerate, nframes, nchannels)
    fid.writeframes(frames)
    fid.close()
    return 0


if __name__ == '__main__':
    import sys
    if len(sys.argv) < 2:
        sys.exit(2)
    sys.exit(create_test_sound(sys.argv[1]))



#! /usr/bin/env python

import sys
import aubio

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('usage: %s <inputfile> <outputfile> [transpose] [samplerate] [hop_size] [mode]' % sys.argv[0])
        print('available modes: default, crispness:0, crispness:1, ... crispness:6')
        sys.exit(1)
    if len(sys.argv) > 3: transpose = float(sys.argv[3])
    else: transpose = 12.
    if len(sys.argv) > 4: samplerate = int(sys.argv[4])
    else: samplerate = 0
    if len(sys.argv) > 5: hop_size = int(sys.argv[5])
    else: hop_size = 64
    if len(sys.argv) > 6: mode = sys.argv[6]
    else: mode = "default"

    source_read = aubio.source(sys.argv[1], samplerate, hop_size)
    if samplerate == 0: samplerate = source_read.samplerate
    sink_out = aubio.sink(sys.argv[2], samplerate)

    pitchshifter = aubio.pitchshift(mode, 1., hop_size, samplerate)
    if transpose: pitchshifter.set_transpose(transpose)

    total_frames, read = 0, hop_size
    transpose_range = 23.9
    while read:
        vec, read = source_read()
        # transpose the samples
        out = pitchshifter(vec)
        # position in the file (between 0. and 1.)
        percent_read = total_frames / float(source_read.duration)
        # variable transpose rate (in semitones)
        transpose = 2 * transpose_range * percent_read - transpose_range
        # set transpose rate
        pitchshifter.set_transpose(transpose)
        # print the transposition
        #print pitchshifter.get_transpose()
        # write the output
        sink_out(out, read)
        total_frames += read

    # end of file, print some results
    outstr = "wrote %.2fs" % (total_frames / float(samplerate))
    outstr += " (%d frames in" % total_frames
    outstr += " %d blocks" % (total_frames // source_read.hop_size)
    outstr += " at %dHz)" % source_read.samplerate
    outstr += " from " + source_read.uri
    outstr += " to " + sink_out.uri
    print(outstr)



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase, assert_equal
import aubio

class aubio_shift_test_case(TestCase):

    def run_shift_ishift(self, n):
        ramp = np.arange(n, dtype=aubio.float_type)
        # construct expected output
        # even length: [5. 6. 7. 8. 9. 0. 1. 2. 3. 4.]
        # odd length: [4. 5. 6. 0. 1. 2. 3.]
        half = n - n//2
        expected = np.concatenate([np.arange(half, n), np.arange(half)])
        # shift in place, returns modified copy
        assert_equal(aubio.shift(ramp), expected)
        # check input was changed as expected
        assert_equal(ramp, expected)
        # construct expected output
        expected = np.arange(n)
        # revert shift in place, returns modifed copy
        assert_equal(aubio.ishift(ramp), expected)
        # check input was shifted back
        assert_equal(ramp, expected)

    def test_can_shift_fvec(self):
        self.run_shift_ishift(10)

    def test_can_shift_fvec_odd(self):
        self.run_shift_ishift(7)

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

import alsaaudio
import numpy as np
import aubio

# constants
samplerate = 44100
win_s = 2048
hop_s = win_s // 2
framesize = hop_s

# set up audio input
recorder = alsaaudio.PCM(type=alsaaudio.PCM_CAPTURE)
recorder.setperiodsize(framesize)
recorder.setrate(samplerate)
recorder.setformat(alsaaudio.PCM_FORMAT_FLOAT_LE)
recorder.setchannels(1)

# create aubio pitch detection (first argument is method, "default" is
# "yinfft", can also be "yin", "mcomb", fcomb", "schmitt").
pitcher = aubio.pitch("default", win_s, hop_s, samplerate)
# set output unit (can be 'midi', 'cent', 'Hz', ...)
pitcher.set_unit("Hz")
# ignore frames under this level (dB)
pitcher.set_silence(-40)

print("Starting to listen, press Ctrl+C to stop")

# main loop
while True:
    try:
        # read data from audio input
        _, data = recorder.read()
        # convert data to aubio float samples
        samples = np.fromstring(data, dtype=aubio.float_type)
        # pitch of current frame
        freq = pitcher(samples)[0]
        # compute energy of current block
        energy = np.sum(samples**2)/len(samples)
        # do something with the results
        print("{:10.4f} {:10.4f}".format(freq,energy))
    except KeyboardInterrupt:
        print("Ctrl+C pressed, exiting")
        break



#! /usr/bin/env python

"""A simple example using aubio.source."""

import sys
import aubio

samplerate = 0  # use original source samplerate
hop_size = 256  # number of frames to read in one block
src = aubio.source(sys.argv[1], samplerate, hop_size)
total_frames = 0

while True:
    samples, read = src()  # read hop_size new samples from source
    total_frames += read   # increment total number of frames
    if read < hop_size:    # end of file reached
        break

fmt_string = "read {:d} frames at {:d}Hz from {:s}"
print(fmt_string.format(total_frames, src.samplerate, src.uri))



#! /usr/bin/env python

"""Create a filterbank from a list of frequencies.

This demo uses `aubio.filterbank.set_triangle_bands` to build a set of
triangular filters from a list of frequencies.

The filterbank coefficients are then modified before being displayed."""

import aubio
import numpy as np
import matplotlib.pyplot as plt

# sampling rate and size of the fft
samplerate = 48000
win_s = 2048

# define a list of custom frequency
freq_list = [60, 80, 200, 400, 800, 1600, 3200, 6400, 12800, 24000]
# number of filters to create
n_filters = len(freq_list) - 2

# create a new filterbank
f = aubio.filterbank(n_filters, win_s)
freqs = aubio.fvec(freq_list)
f.set_triangle_bands(freqs, samplerate)

# get the coefficients from the filterbank
coeffs = f.get_coeffs()
# apply a gain to fifth band
coeffs[4] *= 6.
# load the modified coeffs into the filterbank
f.set_coeffs(coeffs)

# display the band gains in a loglog plot
freqs = np.vstack([np.arange(win_s // 2 + 1) * samplerate / win_s] * n_filters)
plt.title('filterbank built from a list of frequencies\n'
          'The 5th band has been amplified by a factor 6.')
plt.loglog(freqs.T, f.get_coeffs().T, '.-')
plt.xlim([50, samplerate/2])
plt.ylim([1.0e-6, 2.0e-2])
plt.xlabel('log frequency (Hz)')
plt.ylabel('log amplitude')
plt.show()



#! /usr/bin/env python

import sys
import os.path
import aubio


def apply_filter(path, target):
    # open input file, get its samplerate
    s = aubio.source(path)
    samplerate = s.samplerate

    # create an A-weighting filter
    f = aubio.digital_filter(7)
    f.set_a_weighting(samplerate)

    # create output file
    o = aubio.sink(target, samplerate)

    total_frames = 0
    while True:
        # read from source
        samples, read = s()
        # filter samples
        filtered_samples = f(samples)
        # write to sink
        o(filtered_samples, read)
        # count frames read
        total_frames += read
        # end of file reached
        if read < s.hop_size:
            break

    # print some info
    duration = total_frames / float(samplerate)
    input_str = "input: {:s} ({:.2f} s, {:d} Hz)"
    output_str = "output: {:s}, A-weighting filtered ({:d} frames total)"
    print(input_str.format(s.uri, duration, samplerate))
    print(output_str.format(o.uri, total_frames))

if __name__ == '__main__':
    usage = "{:s} <input_file> [output_file]".format(sys.argv[0])
    if not 1 < len(sys.argv) < 4:
        print(usage)
        sys.exit(1)
    if len(sys.argv) < 3:
        input_path = sys.argv[1]
        basename = os.path.splitext(os.path.basename(input_path))[0] + ".wav"
        output_path = "filtered_" + basename
    else:
        input_path, output_path = sys.argv[1:]
    # run function
    apply_filter(input_path, output_path)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from aubio import midi2note
from _tools import parametrize, assert_raises

list_of_known_midis = (
        ( 0, 'C-1' ),
        ( 1, 'C#-1' ),
        ( 38, 'D2' ),
        ( 48, 'C3' ),
        ( 59, 'B3' ),
        ( 60, 'C4' ),
        ( 127, 'G9' ),
        )

class Test_midi2note_good_values(object):

    @parametrize('midi, note', list_of_known_midis)
    def test_midi2note_known_values(self, midi, note):
        " known values are correctly converted "
        assert midi2note(midi) == (note)

class Test_midi2note_wrong_values(object):

    def test_midi2note_negative_value(self):
        " fails when passed a negative value "
        assert_raises(ValueError, midi2note, -2)

    def test_midi2note_large(self):
        " fails when passed a value greater than 127 "
        assert_raises(ValueError, midi2note, 128)

    def test_midi2note_floating_value(self):
        " fails when passed a floating point "
        assert_raises(TypeError, midi2note, 69.2)

    def test_midi2note_character_value(self):
        " fails when passed a value that can not be transformed to integer "
        assert_raises(TypeError, midi2note, "a")

if __name__ == '__main__':
    from _tools import run_module_suite
    run_module_suite()



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase, assert_equal, assert_almost_equal

from aubio import cvec, filterbank, float_type
from utils import array_from_text_file

class aubio_filterbank_test_case(TestCase):

    def test_members(self):
        f = filterbank(40, 512)
        assert_equal ([f.n_filters, f.win_s], [40, 512])

    def test_set_coeffs(self):
        f = filterbank(40, 512)
        r = np.random.random([40, int(512 / 2) + 1]).astype(float_type)
        f.set_coeffs(r)
        assert_equal (r, f.get_coeffs())

    def test_phase(self):
        f = filterbank(40, 512)
        c = cvec(512)
        c.phas[:] = np.pi
        assert_equal( f(c), 0);

    def test_norm(self):
        f = filterbank(40, 512)
        c = cvec(512)
        c.norm[:] = 1
        assert_equal( f(c), 0);

    def test_random_norm(self):
        f = filterbank(40, 512)
        c = cvec(512)
        c.norm[:] = np.random.random((int(512 / 2) + 1,)).astype(float_type)
        assert_equal( f(c), 0)

    def test_random_coeffs(self):
        win_s = 128
        f = filterbank(40, win_s)
        c = cvec(win_s)
        r = np.random.random([40, int(win_s / 2) + 1]).astype(float_type)
        r /= r.sum()
        f.set_coeffs(r)
        c.norm[:] = np.random.random((int(win_s / 2) + 1,)).astype(float_type)
        assert_equal ( f(c) < 1., True )
        assert_equal ( f(c) > 0., True )

    def test_mfcc_coeffs(self):
        f = filterbank(40, 512)
        c = cvec(512)
        f.set_mel_coeffs_slaney(44100)
        c.norm[:] = np.random.random((int(512 / 2) + 1,)).astype(float_type)
        assert_equal ( f(c) < 1., True )
        assert_equal ( f(c) > 0., True )

    def test_mfcc_coeffs_16000(self):
        expected = array_from_text_file('filterbank_mfcc_16000_512.expected')
        f = filterbank(40, 512)
        f.set_mel_coeffs_slaney(16000)
        assert_almost_equal ( expected, f.get_coeffs() )

    def test_mfcc_coeffs_get_coeffs(self):
        f = filterbank(40, 512)
        coeffs = f.get_coeffs()
        self.assertIsInstance(coeffs, np.ndarray)
        assert_equal (coeffs, 0)
        assert_equal (np.shape(coeffs), (40, 512 / 2 + 1))

class aubio_filterbank_wrong_values(TestCase):

    def test_negative_window(self):
        self.assertRaises(ValueError, filterbank, 40, -20)

    def test_negative_filters(self):
        self.assertRaises(ValueError, filterbank, -40, 1024)

    def test_filterbank_long_cvec(self):
        f = filterbank(40, 512)
        with self.assertRaises(ValueError):
            f(cvec(1024))

    def test_filterbank_short_cvec(self):
        f = filterbank(40, 512)
        with self.assertRaises(ValueError):
            f(cvec(256))

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

import numpy as np
import aubio

def build_sinusoid(length, freqs, samplerate):
    return np.sin( 2. * np.pi * np.arange(length) * freqs / samplerate).astype(aubio.float_type)

def run_pitch(p, input_vec):
    cands = []
    for vec_slice in input_vec.reshape((-1, p.hop_size)):
        a = p(vec_slice)[0]
        cands.append(a)
    return cands

methods = ['default', 'schmitt', 'fcomb', 'mcomb', 'yin', 'yinfft']

cands = {}
buf_size = 2048
hop_size = 512
samplerate = 44100
sin_length = (samplerate * 10) % 512 * 512
freqs = np.zeros(sin_length)

partition = sin_length // 8
pointer = 0

pointer += partition
freqs[pointer: pointer + partition] = 440

pointer += partition
pointer += partition
freqs[ pointer : pointer + partition ] = 740

pointer += partition
freqs[ pointer : pointer + partition ] = 1480

pointer += partition
pointer += partition
freqs[ pointer : pointer + partition ] = 400 + 5 * np.random.random(sin_length//8)

a = build_sinusoid(sin_length, freqs, samplerate)

for method in methods:
    p = aubio.pitch(method, buf_size, hop_size, samplerate)
    cands[method] = run_pitch(p, a)
    print(method)
    print(cands[method])

print("done computing")

if 1:
    import matplotlib.pyplot as plt

    # times
    ramp = np.arange(0, sin_length / hop_size).astype('float') * hop_size / samplerate

    # plot each result
    for method in methods:
        plt.plot(ramp, cands[method], '.-', label=method)

    # plot ground truth
    ramp = np.arange(0, sin_length).astype('float') / samplerate
    plt.plot(ramp, freqs, ':', label = 'ground truth')

    plt.legend(loc='upper left')

    plt.xlabel('time (s)')
    plt.ylabel('frequency (Hz)')
    plt.ylim([0,2000])
    plt.show()



#! /usr/bin/env python


from numpy.testing import TestCase, assert_equal
from aubio import source
from utils import list_all_sounds, parse_file_samplerate
import unittest
from _tools import assert_raises, assert_equal, assert_warns
from _tools import parametrize, skipTest

list_of_sounds = list_all_sounds('sounds')
samplerates = [0, 44100, 8000, 32000]
hop_sizes = [512, 1024, 64]

default_test_sound = len(list_of_sounds) and list_of_sounds[0] or None

all_params = []
for soundfile in list_of_sounds:
    for hop_size in hop_sizes:
        for samplerate in samplerates:
            all_params.append((hop_size, samplerate, soundfile))

no_sounds_msg = "no test sounds, add some in 'python/tests/sounds/'!"

_debug = False


class Test_aubio_source_test_case(TestCase):

    def setUp(self):
        if not default_test_sound:
            skipTest(no_sounds_msg)

    def test_close_file(self):
        samplerate = 0 # use native samplerate
        hop_size = 256
        f = source(default_test_sound, samplerate, hop_size)
        f.close()

    def test_close_file_twice(self):
        samplerate = 0 # use native samplerate
        hop_size = 256
        f = source(default_test_sound, samplerate, hop_size)
        f.close()
        f.close()

    def test_read_after_close(self):
        samplerate = 0 # use native samplerate
        hop_size = 256
        f = source(default_test_sound, samplerate, hop_size)
        read, frames = f()
        f.close()
        with assert_raises(RuntimeError):
            read, frames = f()
        with assert_raises(RuntimeError):
            read, frames = f.do_multi()


class Test_aubio_source_read(object):

    def read_from_source(self, f):
        total_frames = 0
        while True:
            samples , read = f()
            total_frames += read
            if read < f.hop_size:
                assert_equal(samples[read:], 0)
                break
        if _debug:
            result_str = "read {:.2f}s ({:d} frames"
            result_str += " in {:d} blocks at {:d}Hz) from {:s}"
            result_params = total_frames / float(f.samplerate), total_frames, \
                    total_frames//f.hop_size, f.samplerate, f.uri
            print (result_str.format(*result_params))
        return total_frames

    @parametrize('hop_size, samplerate, soundfile', all_params)
    def test_samplerate_hopsize(self, hop_size, samplerate, soundfile):
        orig_samplerate = parse_file_samplerate(soundfile)
        try:
            if orig_samplerate is not None and orig_samplerate < samplerate:
                # upsampling should emit a warning
                with assert_warns(UserWarning):
                    f = source(soundfile, samplerate, hop_size)
            else:
                f = source(soundfile, samplerate, hop_size)
        except RuntimeError as e:
            err_msg = 'failed opening with hop_s={:d}, samplerate={:d} ({:s})'
            skipTest(err_msg.format(hop_size, samplerate, str(e)))
        assert f.samplerate != 0
        read_frames = self.read_from_source(f)
        if 'f_' in soundfile and samplerate == 0:
            import re
            f = re.compile(r'.*_\([0:9]*f\)_.*')
            match_f = re.findall('([0-9]*)f_', soundfile)
            if len(match_f) == 1:
                expected_frames = int(match_f[0])
                assert_equal(expected_frames, read_frames)

    @parametrize('p', list_of_sounds)
    def test_samplerate_none(self, p):
        f = source(p)
        assert f.samplerate != 0
        self.read_from_source(f)

    @parametrize('p', list_of_sounds)
    def test_samplerate_0(self, p):
        f = source(p, 0)
        assert f.samplerate != 0
        self.read_from_source(f)

    @parametrize('p', list_of_sounds)
    def test_zero_hop_size(self, p):
        f = source(p, 0, 0)
        assert f.samplerate != 0
        assert f.hop_size != 0
        self.read_from_source(f)

    @parametrize('p', list_of_sounds)
    def test_seek_to_half(self, p):
        from random import randint
        f = source(p, 0, 0)
        assert f.samplerate != 0
        assert f.hop_size != 0
        a = self.read_from_source(f)
        c = randint(0, a)
        f.seek(c)
        b = self.read_from_source(f)
        assert a == b + c

    @parametrize('p', list_of_sounds)
    def test_duration(self, p):
        total_frames = 0
        f = source(p)
        duration = f.duration
        while True:
            _, read = f()
            total_frames += read
            if read < f.hop_size: break
        assert_equal (duration, total_frames)


class Test_aubio_source_wrong_params(object):

    def test_wrong_file(self):
        with assert_raises(RuntimeError):
            source('path_to/unexisting file.mp3')

@unittest.skipIf(default_test_sound is None, no_sounds_msg)
class Test_aubio_source_wrong_params_with_file(TestCase):

    def test_wrong_samplerate(self):
        with assert_raises(ValueError):
            source(default_test_sound, -1)

    def test_wrong_hop_size(self):
        with assert_raises(ValueError):
            source(default_test_sound, 0, -1)

    def test_wrong_channels(self):
        with assert_raises(ValueError):
            source(default_test_sound, 0, 0, -1)

    def test_wrong_seek(self):
        f = source(default_test_sound)
        with assert_raises(ValueError):
            f.seek(-1)

    def test_wrong_seek_too_large(self):
        f = source(default_test_sound)
        try:
            with assert_raises(ValueError):
                f.seek(f.duration + f.samplerate * 10)
        except:
            skipTest('seeking after end of stream failed raising ValueError')

class Test_aubio_source_readmulti(Test_aubio_source_read):

    def read_from_source(self, f):
        total_frames = 0
        while True:
            samples, read = f.do_multi()
            total_frames += read
            if read < f.hop_size:
                assert_equal(samples[:,read:], 0)
                break
        if _debug:
            result_str = "read {:.2f}s ({:d} frames in {:d} channels"
            result_str += " and {:d} blocks at {:d}Hz) from {:s}"
            result_params = total_frames / float(f.samplerate), total_frames, \
                    f.channels, int(total_frames/f.hop_size), \
                    f.samplerate, f.uri
            print (result_str.format(*result_params))
        return total_frames

class Test_aubio_source_with(object):

    @parametrize('filename', list_of_sounds)
    def test_read_from_mono(self, filename):
        total_frames = 0
        hop_size = 2048
        with source(filename, 0, hop_size) as input_source:
            assert_equal(input_source.hop_size, hop_size)
            #assert_equal(input_source.samplerate, samplerate)
            total_frames = 0
            for frames in input_source:
                total_frames += frames.shape[-1]
            # check we read as many samples as we expected
            assert_equal(total_frames, input_source.duration)

if __name__ == '__main__':
    from _tools import run_module_suite
    run_module_suite()



#! /usr/bin/env python

import sys
from aubio import source, sink, pvoc

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('usage: %s <inputfile> <outputfile>' % sys.argv[0])
        sys.exit(1)
    samplerate = 44100
    f = source(sys.argv[1], samplerate, 256)
    g = sink(sys.argv[2], samplerate)
    total_frames, read = 0, 256

    win_s = 512                          # fft size
    hop_s = win_s // 2                   # hop size
    pv = pvoc(win_s, hop_s)              # phase vocoder

    while read:
        samples, read = f()
        spectrum = pv(samples)           # compute spectrum
        #spectrum.norm *= .8             # reduce amplitude a bit
        spectrum.phas[:] = 0.            # zero phase
        new_samples = pv.rdo(spectrum)   # compute modified samples
        g(new_samples, read)             # write to output
        total_frames += read

    format_str = "read {:d} samples from {:s}, written to {:s}"
    print(format_str.format(total_frames, f.uri, g.uri))



#! /usr/bin/env python

from aubio import filterbank
from numpy import arange, vstack

win_s = 8192
samplerate = 16000

f = filterbank(40, win_s)
f.set_mel_coeffs_slaney(samplerate)

from pylab import loglog, title, show, xlim, ylim, xlabel, ylabel
xlim([0,samplerate / 2])
times = vstack([arange(win_s // 2 + 1) * samplerate / win_s] * 40)
loglog(times.T, f.get_coeffs().T, '.-')
title('Mel frequency bands coefficients')
xlim([100, 7500])
ylim([1.0e-3, 2.0e-2])
xlabel('Frequency (Hz)')
ylabel('Amplitude')
show()



#! /usr/bin/env python

import sys
from aubio import onset, source
from numpy import hstack, zeros

win_s = 512                 # fft size
hop_s = win_s // 2          # hop size

if len(sys.argv) < 2:
    print("Usage: %s <filename> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]

samplerate = 0
if len( sys.argv ) > 2: samplerate = int(sys.argv[2])

s = source(filename, samplerate, hop_s)
samplerate = s.samplerate
o = onset("default", win_s, hop_s, samplerate)

# list of onsets, in samples
onsets = []

# storage for plotted data
desc = []
tdesc = []
allsamples_max = zeros(0,)
downsample = 2  # to plot n samples / hop_s

# total number of frames read
total_frames = 0
while True:
    samples, read = s()
    if o(samples):
        print("%f" % (o.get_last_s()))
        onsets.append(o.get_last())
    # keep some data to plot it later
    new_maxes = (abs(samples.reshape(hop_s//downsample, downsample))).max(axis=0)
    allsamples_max = hstack([allsamples_max, new_maxes])
    desc.append(o.get_descriptor())
    tdesc.append(o.get_thresholded_descriptor())
    total_frames += read
    if read < hop_s: break

if 1:
    # do plotting
    import matplotlib.pyplot as plt
    allsamples_max = (allsamples_max > 0) * allsamples_max
    allsamples_max_times = [ float(t) * hop_s / downsample / samplerate for t in range(len(allsamples_max)) ]
    plt1 = plt.axes([0.1, 0.75, 0.8, 0.19])
    plt2 = plt.axes([0.1, 0.1, 0.8, 0.65], sharex = plt1)
    plt.rc('lines',linewidth='.8')
    plt1.plot(allsamples_max_times,  allsamples_max, '-b')
    plt1.plot(allsamples_max_times, -allsamples_max, '-b')
    for stamp in onsets:
        stamp /= float(samplerate)
        plt1.plot([stamp, stamp], [-1., 1.], '-r')
    plt1.axis(xmin = 0., xmax = max(allsamples_max_times) )
    plt1.xaxis.set_visible(False)
    plt1.yaxis.set_visible(False)
    desc_times = [ float(t) * hop_s / samplerate for t in range(len(desc)) ]
    desc_max = max(desc) if max(desc) != 0 else 1.
    desc_plot = [d / desc_max for d in desc]
    plt2.plot(desc_times, desc_plot, '-g')
    tdesc_plot = [d / desc_max for d in tdesc]
    for stamp in onsets:
        stamp /= float(samplerate)
        plt2.plot([stamp, stamp], [min(tdesc_plot), max(desc_plot)], '-r')
    plt2.plot(desc_times, tdesc_plot, '-y')
    plt2.axis(ymin = min(tdesc_plot), ymax = max(desc_plot))
    plt.xlabel('time (s)')
    #plt.savefig('/tmp/t.png', dpi=200)
    plt.show()



#! /usr/bin/env python
# -*- coding: utf8 -*-

""" Pure python implementation of the sum of squared difference

    sqd_yin: original sum of squared difference [0]
        d_t(tau) = x ⊗ kernel
    sqd_yinfast: sum of squared diff using complex domain [0]
    sqd_yinfftslow: tappered squared diff [1]
    sqd_yinfft: modified squared diff using complex domain [1]

[0]:http://audition.ens.fr/adc/pdf/2002_JASA_YIN.pdf
[1]:https://aubio.org/phd/
"""

import sys
import numpy as np
import matplotlib.pyplot as plt

def sqd_yin(samples):
    """ compute original sum of squared difference

    Brute-force computation (cost o(N**2), slow)."""
    B = len(samples)
    W = B//2
    yin = np.zeros(W)
    for j in range(W):
        for tau in range(1, W):
            yin[tau] += (samples[j] - samples[j+tau])**2
    return yin

def sqd_yinfast(samples):
    """ compute approximate sum of squared difference

    Using complex convolution (fast, cost o(n*log(n)) )"""
    # yin_t(tau) = (r_t(0) + r_(t+tau)(0)) - 2r_t(tau)
    B = len(samples)
    W = B//2
    yin = np.zeros(W)
    sqdiff = np.zeros(W)
    kernel = np.zeros(B)
    # compute r_(t+tau)(0)
    squares = samples**2
    for tau in range(W):
        sqdiff[tau] = squares[tau:tau+W].sum()
    # add r_t(0)
    sqdiff += sqdiff[0]
    # compute r_t(tau) using kernel convolution in complex domain
    samples_fft = np.fft.fft(samples)
    kernel[1:W+1] = samples[W-1::-1] # first half, reversed
    kernel_fft = np.fft.fft(kernel)
    r_t_tau = np.fft.ifft(samples_fft * kernel_fft).real[W:]
    # compute yin_t(tau)
    yin = sqdiff - 2 * r_t_tau
    return yin

def sqd_yintapered(samples):
    """ compute tappered sum of squared difference

    Brute-force computation (cost o(N**2), slow)."""
    B = len(samples)
    W = B//2
    yin = np.zeros(W)
    for tau in range(1, W):
        for j in range(W - tau):
            yin[tau] += (samples[j] - samples[j+tau])**2
    return yin

def sqd_yinfft(samples):
    """ compute yinfft modified sum of squared differences

    Very fast, improved performance in transients.

    FIXME: biased."""
    B = len(samples)
    W = B//2
    yin = np.zeros(W)
    def hanningz(W):
        return .5 * (1. - np.cos(2. * np.pi * np.arange(W) / W))
    #win = np.ones(B)
    win = hanningz(B)
    sqrmag = np.zeros(B)
    fftout = np.fft.fft(win*samples)
    sqrmag[0] = fftout[0].real**2
    for l in range(1, W):
        sqrmag[l] = fftout[l].real**2 + fftout[l].imag**2
        sqrmag[B-l] = sqrmag[l]
    sqrmag[W] = fftout[W].real**2
    fftout = np.fft.fft(sqrmag)
    sqrsum = 2.*sqrmag[:W + 1].sum()
    yin[0] = 0
    yin[1:] = sqrsum - fftout.real[1:W]
    return yin / B

def cumdiff(yin):
    """ compute the cumulative mean normalized difference """
    W = len(yin)
    yin[0] = 1.
    cumsum = 0.
    for tau in range(1, W):
        cumsum += yin[tau]
        if cumsum != 0:
            yin[tau] *= tau/cumsum
        else:
            yin[tau] = 1
    return yin

def compute_all(x):
    import time
    now = time.time()

    yin     = sqd_yin(x)
    t1 = time.time()
    print ("yin took %.2fms" % ((t1-now) * 1000.))

    yinfast = sqd_yinfast(x)
    t2 = time.time()
    print ("yinfast took: %.2fms" % ((t2-t1) * 1000.))

    yintapered = sqd_yintapered(x)
    t3 = time.time()
    print ("yintapered took: %.2fms" % ((t3-t2) * 1000.))

    yinfft  = sqd_yinfft(x)
    t4 = time.time()
    print ("yinfft took: %.2fms" % ((t4-t3) * 1000.))

    return yin, yinfast, yintapered, yinfft

def plot_all(yin, yinfast, yintapered, yinfft):
    fig, axes = plt.subplots(nrows=2, ncols=2, sharex=True, sharey='col')

    axes[0, 0].plot(yin, label='yin')
    axes[0, 0].plot(yintapered, label='yintapered')
    axes[0, 0].set_ylim(bottom=0)
    axes[0, 0].legend()
    axes[1, 0].plot(yinfast, '-', label='yinfast')
    axes[1, 0].plot(yinfft, label='yinfft')
    axes[1, 0].legend()

    axes[0, 1].plot(cumdiff(yin), label='yin')
    axes[0, 1].plot(cumdiff(yintapered), label='yin tapered')
    axes[0, 1].set_ylim(bottom=0)
    axes[0, 1].legend()
    axes[1, 1].plot(cumdiff(yinfast), '-', label='yinfast')
    axes[1, 1].plot(cumdiff(yinfft), label='yinfft')
    axes[1, 1].legend()

    fig.tight_layout()

testfreqs = [441., 800., 10000., 40.]

if len(sys.argv) > 1:
    testfreqs = map(float,sys.argv[1:])

for f in testfreqs:
    print ("Comparing yin implementations for sine wave at %.fHz" % f)
    samplerate = 44100.
    win_s = 4096

    x = np.cos(2.*np.pi * np.arange(win_s) * f / samplerate)

    n_times = 1#00
    for n in range(n_times):
        yin, yinfast, yinfftslow, yinfft = compute_all(x)
    if 0: # plot difference
        plt.plot(yin-yinfast)
        plt.tight_layout()
        plt.show()
    if 1:
        plt.plot(yinfftslow-yinfft)
        plt.tight_layout()
        plt.show()
    plot_all(yin, yinfast, yinfftslow, yinfft)
plt.show()



#! /usr/bin/env python

from numpy.testing import TestCase
import aubio.cmd

class aubio_cmd(TestCase):

    def setUp(self):
        self.a_parser = aubio.cmd.aubio_parser()

    def test_default_creation(self):
        try:
            assert self.a_parser.parse_args(['-V']).show_version
        except SystemExit:
            url = 'https://bugs.python.org/issue9253'
            self.skipTest('subcommand became optional in py3, see %s' % url)

class aubio_cmd_utils(TestCase):

    def test_samples2seconds(self):
        self.assertEqual(aubio.cmd.samples2seconds(3200, 32000),
                "0.100000\t")

    def test_samples2milliseconds(self):
        self.assertEqual(aubio.cmd.samples2milliseconds(3200, 32000),
                "100.000000\t")

    def test_samples2samples(self):
        self.assertEqual(aubio.cmd.samples2samples(3200, 32000),
                "3200\t")

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

""" A simple demo using aubio and pyaudio to play beats in real time

Note you will need to have pyaudio installed: `pip install pyaudio`.

Examples:
    ./demo_tapthebeat.py ~/Music/track1.ogg

When compiled with ffmpeg/libav, you should be able to open remote streams. For
instance using youtube-dl (`pip install youtube-dl`):

    ./demo_tapthebeat.py `youtube-dl -xg https://youtu.be/zZbM9n9j3_g`

"""

import sys
import time
import pyaudio
import aubio
import numpy as np

win_s = 1024                # fft size
hop_s = win_s // 2          # hop size

# parse command line arguments
if len(sys.argv) < 2:
    print("Usage: %s <filename> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]

samplerate = 0
if len( sys.argv ) > 2: samplerate = int(sys.argv[2])

# create aubio source
a_source = aubio.source(filename, samplerate, hop_s)
samplerate = a_source.samplerate

# create aubio tempo detection
a_tempo = aubio.tempo("default", win_s, hop_s, samplerate)

# create a simple click sound
click = 0.7 * np.sin(2. * np.pi * np.arange(hop_s) / hop_s * samplerate / 3000.)

# pyaudio callback
def pyaudio_callback(_in_data, _frame_count, _time_info, _status):
    samples, read = a_source()
    is_beat = a_tempo(samples)
    if is_beat:
        samples += click
        #print ('tick') # avoid print in audio callback
    audiobuf = samples.tobytes()
    if read < hop_s:
        return (audiobuf, pyaudio.paComplete)
    return (audiobuf, pyaudio.paContinue)

# create pyaudio stream with frames_per_buffer=hop_s and format=paFloat32
p = pyaudio.PyAudio()
pyaudio_format = pyaudio.paFloat32
frames_per_buffer = hop_s
n_channels = 1
stream = p.open(format=pyaudio_format, channels=n_channels, rate=samplerate,
        output=True, frames_per_buffer=frames_per_buffer,
        stream_callback=pyaudio_callback)

# start pyaudio stream
stream.start_stream()

# wait for stream to finish
while stream.is_active():
    time.sleep(0.1)

# stop pyaudio stream
stream.stop_stream()
stream.close()
# close pyaudio
p.terminate()



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase, assert_equal, assert_almost_equal
from _tools import assert_warns

from aubio import fvec, cvec, filterbank, float_type

class aubio_filterbank_mel_test_case(TestCase):

    def test_slaney(self):
        f = filterbank(40, 512)
        f.set_mel_coeffs_slaney(16000)
        a = f.get_coeffs()
        assert_equal(np.shape (a), (40, 512/2 + 1) )

    def test_other_slaney(self):
        f = filterbank(40, 512*2)
        f.set_mel_coeffs_slaney(44100)
        self.assertIsInstance(f.get_coeffs(), np.ndarray)
        #print "sum is", sum(sum(a))
        for win_s in [256, 512, 1024, 2048, 4096]:
            f = filterbank(40, win_s)
            f.set_mel_coeffs_slaney(32000)
            #print "sum is", sum(sum(a))
            self.assertIsInstance(f.get_coeffs(), np.ndarray)

    def test_triangle_freqs_zeros(self):
        f = filterbank(9, 1024)
        freq_list = [40, 80, 200, 400, 800, 1600, 3200, 6400, 12800, 15000, 24000]
        freqs = np.array(freq_list, dtype = float_type)
        f.set_triangle_bands(freqs, 48000)
        assert_equal ( f(cvec(1024)), 0)
        self.assertIsInstance(f.get_coeffs(), np.ndarray)

    def test_triangle_freqs_ones(self):
        f = filterbank(9, 1024)
        freq_list = [40, 80, 200, 400, 800, 1600, 3200, 6400, 12800, 15000, 24000]
        freqs = np.array(freq_list, dtype = float_type)
        f.set_triangle_bands(freqs, 48000)
        self.assertIsInstance(f.get_coeffs(), np.ndarray)
        spec = cvec(1024)
        spec.norm[:] = 1
        assert_almost_equal ( f(spec),
                [ 0.02070313, 0.02138672, 0.02127604, 0.02135417,
                    0.02133301, 0.02133301, 0.02133311, 0.02133334, 0.02133345])

    def test_triangle_freqs_with_zeros(self):
        """make sure set_triangle_bands works when list starts with 0"""
        freq_list = [0, 40, 80]
        freqs = np.array(freq_list, dtype = float_type)
        f = filterbank(len(freqs)-2, 1024)
        f.set_triangle_bands(freqs, 48000)
        assert_equal ( f(cvec(1024)), 0)
        self.assertIsInstance(f.get_coeffs(), np.ndarray)

    def test_triangle_freqs_with_wrong_negative(self):
        """make sure set_triangle_bands fails when list contains a negative"""
        freq_list = [-10, 0, 80]
        f = filterbank(len(freq_list)-2, 1024)
        with self.assertRaises(ValueError):
            f.set_triangle_bands(fvec(freq_list), 48000)

    def test_triangle_freqs_with_wrong_ordering(self):
        """make sure set_triangle_bands fails when list not ordered"""
        freq_list = [0, 80, 40]
        f = filterbank(len(freq_list)-2, 1024)
        with self.assertRaises(ValueError):
            f.set_triangle_bands(fvec(freq_list), 48000)

    def test_triangle_freqs_with_large_freq(self):
        """make sure set_triangle_bands warns when freq > nyquist"""
        samplerate = 22050
        freq_list = [0, samplerate//4, samplerate // 2 + 1]
        f = filterbank(len(freq_list)-2, 1024)
        with assert_warns(UserWarning):
            f.set_triangle_bands(fvec(freq_list), samplerate)

    def test_triangle_freqs_with_not_enough_filters(self):
        """make sure set_triangle_bands warns when not enough filters"""
        samplerate = 22050
        freq_list = [0, 100, 1000, 4000, 8000, 10000]
        f = filterbank(len(freq_list)-3, 1024)
        with assert_warns(UserWarning):
            f.set_triangle_bands(fvec(freq_list), samplerate)

    def test_triangle_freqs_with_too_many_filters(self):
        """make sure set_triangle_bands warns when too many filters"""
        samplerate = 22050
        freq_list = [0, 100, 1000, 4000, 8000, 10000]
        f = filterbank(len(freq_list)-1, 1024)
        with assert_warns(UserWarning):
            f.set_triangle_bands(fvec(freq_list), samplerate)

    def test_triangle_freqs_with_double_value(self):
        """make sure set_triangle_bands works with 2 duplicate freqs"""
        samplerate = 22050
        freq_list = [0, 100, 1000, 4000, 4000, 4000, 10000]
        f = filterbank(len(freq_list)-2, 1024)
        with assert_warns(UserWarning):
            f.set_triangle_bands(fvec(freq_list), samplerate)

    def test_triangle_freqs_with_triple(self):
        """make sure set_triangle_bands works with 3 duplicate freqs"""
        samplerate = 22050
        freq_list = [0, 100, 1000, 4000, 4000, 4000, 10000]
        f = filterbank(len(freq_list)-2, 1024)
        with assert_warns(UserWarning):
            f.set_triangle_bands(fvec(freq_list), samplerate)


    def test_triangle_freqs_without_norm(self):
        """make sure set_triangle_bands works without """
        samplerate = 22050
        freq_list = fvec([0, 100, 1000, 10000])
        f = filterbank(len(freq_list) - 2, 1024)
        f.set_norm(0)
        f.set_triangle_bands(freq_list, samplerate)
        expected = f.get_coeffs()
        f.set_norm(1)
        f.set_triangle_bands(fvec(freq_list), samplerate)
        assert_almost_equal(f.get_coeffs().T,
                expected.T * 2. / (freq_list[2:] - freq_list[:-2]))

    def test_triangle_freqs_wrong_norm(self):
        f = filterbank(10, 1024)
        with self.assertRaises(ValueError):
            f.set_norm(-1)

    def test_triangle_freqs_with_power(self):
        f = filterbank(9, 1024)
        freqs = fvec([40, 80, 200, 400, 800, 1600, 3200, 6400, 12800, 15000,
            24000])
        f.set_power(2)
        f.set_triangle_bands(freqs, 48000)
        spec = cvec(1024)
        spec.norm[:] = .1
        expected = fvec([0.02070313, 0.02138672, 0.02127604, 0.02135417,
            0.02133301, 0.02133301, 0.02133311, 0.02133334, 0.02133345])
        expected /= 100.
        assert_almost_equal(f(spec), expected)

    def test_mel_coeffs(self):
        f = filterbank(40, 1024)
        f.set_mel_coeffs(44100, 0, 44100 / 2)

    def test_zero_fmax(self):
        f = filterbank(40, 1024)
        f.set_mel_coeffs(44100, 0, 0)

    def test_wrong_mel_coeffs(self):
        f = filterbank(40, 1024)
        with self.assertRaises(ValueError):
            f.set_mel_coeffs_slaney(0)
        with self.assertRaises(ValueError):
            f.set_mel_coeffs(44100, 0, -44100 / 2)
        with self.assertRaises(ValueError):
            f.set_mel_coeffs(44100, -0.1, 44100 / 2)
        with self.assertRaises(ValueError):
            f.set_mel_coeffs(-44100, 0.1, 44100 / 2)
        with self.assertRaises(ValueError):
            f.set_mel_coeffs_htk(-1, 0, 0)

    def test_mel_coeffs_htk(self):
        f = filterbank(40, 1024)
        f.set_mel_coeffs_htk(44100, 0, 44100 / 2)


if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

import sys
from aubio import source, sink, pvoc, tss

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('usage: %s <inputfile> <outputfile_transient> <outputfile_steady>' % sys.argv[0])
        sys.exit(1)

    samplerate = 44100
    win_s = 1024       # fft size
    hop_s = win_s // 8 # block size

    f = source(sys.argv[1], samplerate, hop_s)
    g = sink(sys.argv[2], samplerate)
    h = sink(sys.argv[3], samplerate)

    pva = pvoc(win_s, hop_s)    # a phase vocoder
    pvb = pvoc(win_s, hop_s)    # another phase vocoder
    t = tss(win_s, hop_s)       # transient steady state separation

    t.set_threshold(0.01)
    t.set_alpha(3.)
    t.set_beta(4.)

    read = hop_s

    while read:
        samples, read = f()               # read file
        spec = pva(samples)               # compute spectrum
        trans_spec, stead_spec = t(spec)  # transient steady-state separation
        transients = pva.rdo(trans_spec)  # overlap-add synthesis of transients
        steadstate = pvb.rdo(stead_spec)  # overlap-add synthesis of steady states
        g(transients, read)               # write transients to output
        h(steadstate, read)               # write steady states to output

    del f, g, h                           # finish writing the files now
    sys.exit(0)

    from demo_spectrogram import get_spectrogram
    from pylab import subplot, show
    subplot(311)
    get_spectrogram(sys.argv[1])
    subplot(312)
    get_spectrogram(sys.argv[2])
    subplot(313)
    get_spectrogram(sys.argv[3])
    show()



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase, assert_equal, assert_allclose
from aubio import cvec, fvec, float_type

wrong_type = 'float32' if float_type == 'float64' else 'float64'

class aubio_cvec_test_case(TestCase):

    def test_vector_created_with_zeroes(self):
        a = cvec(10)
        assert_equal(a.norm.shape[0], 10 // 2 + 1)
        assert_equal(a.phas.shape[0], 10 // 2 + 1)
        assert_equal(a.norm, 0.)
        assert_equal(a.phas, 0.)

    def test_vector_assign_element(self):
        a = cvec()
        a.norm[0] = 1
        assert_equal(a.norm[0], 1)
        a.phas[0] = 1
        assert_equal(a.phas[0], 1)

    def test_vector_assign_element_end(self):
        a = cvec()
        a.norm[-1] = 1
        assert_equal(a.norm[-1], 1)
        assert_equal(a.norm[len(a.norm)-1], 1)
        a.phas[-1] = 1
        assert_equal(a.phas[-1], 1)
        assert_equal(a.phas[len(a.phas)-1], 1)

    def test_assign_cvec_norm_slice(self):
        spec = cvec(1024)
        spec.norm[40:100] = 100
        assert_equal(spec.norm[0:40], 0)
        assert_equal(spec.norm[40:100], 100)
        assert_equal(spec.norm[100:-1], 0)
        assert_equal(spec.phas, 0)

    def test_assign_cvec_phas_slice(self):
        spec = cvec(1024)
        spec.phas[39:-1] = -np.pi
        assert_equal(spec.phas[0:39], 0)  # exact comparison ok for zero
        assert_allclose(spec.phas[39:-1], -np.pi)  # use allclose for floating point - numpy 2 is stricter
        assert_equal(spec.norm, 0)  # exact comparison ok for zero

    def test_assign_cvec_with_other_cvec(self):
        """ check dest cvec is still reachable after source was deleted """
        spec = cvec(1024)
        a = np.random.rand(1024//2+1).astype(float_type)
        b = np.random.rand(1024//2+1).astype(float_type)
        spec.norm = a
        spec.phas = b
        new_spec = spec
        del spec
        assert_equal(a, new_spec.norm)
        assert_equal(b, new_spec.phas)
        assert_equal(id(a), id(new_spec.norm))
        assert_equal(id(b), id(new_spec.phas))

    def test_pass_to_numpy(self):
        spec = cvec(1024)
        norm = spec.norm
        phas = spec.phas
        del spec
        new_spec = cvec(1024)
        new_spec.norm = norm
        new_spec.phas = phas
        assert_equal(norm, new_spec.norm)
        assert_equal(phas, new_spec.phas)
        assert_equal(id(norm), id(new_spec.norm))
        assert_equal(id(phas), id(new_spec.phas))
        del norm
        del phas
        assert_equal(new_spec.norm, 0.)
        assert_equal(new_spec.phas, 0.)
        del new_spec

    def test_assign_norm_too_large(self):
        a = cvec(512)
        b = fvec(512//2+1 + 4)
        with self.assertRaises(ValueError):
            a.norm = b

    def test_assign_norm_too_small(self):
        a = cvec(512)
        b = fvec(512//2+1 - 4)
        with self.assertRaises(ValueError):
            a.norm = b

    def test_assign_phas_too_large(self):
        a = cvec(512)
        b = fvec(512//2+1 + 4)
        with self.assertRaises(ValueError):
            a.phas = b

    def test_assign_phas_too_small(self):
        a = cvec(512)
        b = fvec(512//2+1 - 4)
        with self.assertRaises(ValueError):
            a.phas = b

    def test_cvec_repr(self):
        win_s = 512
        c = cvec(win_s)
        expected_repr = "aubio cvec of {:d} elements".format(win_s//2+1)
        self.assertEqual(repr(c), expected_repr)

class aubio_cvec_wrong_norm_input(TestCase):

    def test_wrong_length(self):
        with self.assertRaises(ValueError):
            cvec(-1)

    def test_set_norm_with_scalar(self):
        a = cvec(512)
        with self.assertRaises(ValueError):
            a.norm = 1

    def test_set_norm_with_scalar_array(self):
        a = cvec(512)
        with self.assertRaises(ValueError):
            a.norm = np.ndarray(1, dtype = 'int')

    def test_set_norm_with_int_array(self):
        a = cvec(512)
        with self.assertRaises(ValueError):
            a.norm = np.zeros(512//2+1, dtype = 'int')

    def test_set_norm_with_wrong_float_array(self):
        a = cvec(512)
        with self.assertRaises(ValueError):
            a.norm = np.zeros(512//2+1, dtype = wrong_type)

    def test_set_norm_with_wrong_2d_array(self):
        a = cvec(512)
        with self.assertRaises(ValueError):
            a.norm = np.zeros((512//2+1, 2), dtype = float_type)

if __name__ == '__main__':
    from unittest import main
    main()



import sys
import os
import subprocess
import glob
from gen_code import MappedObject

header = os.path.join('src', 'aubio.h')
output_path = os.path.join('python', 'gen')

source_header = """// this file is generated! do not modify
#include "aubio-types.h"
"""

default_skip_objects = [
    # already in ext/
    'fft',
    'pvoc',
    'filter',
    'filterbank',
    # AUBIO_UNSTABLE
    'hist',
    'parameter',
    'scale',
    'beattracking',
    'resampler',
    'peakpicker',
    'pitchfcomb',
    'pitchmcomb',
    'pitchschmitt',
    'pitchspecacf',
    'pitchyin',
    'pitchyinfft',
    'pitchyinfast',
    'sink',
    'sink_apple_audio',
    'sink_sndfile',
    'sink_wavwrite',
    #'mfcc',
    'source',
    'source_apple_audio',
    'source_sndfile',
    'source_avcodec',
    'source_wavread',
    #'sampler',
    'audio_unit',
    'spectral_whitening',
    'timestretch', # TODO fix parsing of uint_t *read in _do
]


def get_preprocessor():
    """Get the C preprocessor command without using distutils."""
    # Try to use the CC environment variable, otherwise fall back to common compilers
    cc = os.environ.get('CC', None)
    
    if cc:
        # CC is set, use it
        cpp_cmd = cc.split()
    else:
        # Try to find an available C compiler
        # On all platforms, try gcc/clang/cc first, then cl.exe as last resort
        compilers_to_try = ['gcc', 'clang', 'cc']
        if os.name == 'nt':
            # On Windows, also try cl.exe as last resort
            compilers_to_try.append('cl.exe')
        
        cpp_cmd = None
        for compiler in compilers_to_try:
            try:
                subprocess.run([compiler, '--version' if compiler != 'cl.exe' else '/?'], 
                             stdout=subprocess.PIPE, 
                             stderr=subprocess.PIPE, 
                             check=True,
                             timeout=5)
                cpp_cmd = [compiler]
                break
            except (subprocess.CalledProcessError, FileNotFoundError, subprocess.TimeoutExpired):
                continue
        
        if cpp_cmd is None:
            raise RuntimeError("No C compiler found. Please install gcc, clang, or MSVC and ensure it's in PATH.")
    
    # Add preprocessor flag
    if 'cl.exe' in cpp_cmd[0] or 'cl' == cpp_cmd[0]:
        cpp_cmd += ['/E']  # MSVC preprocessor flag
    else:
        cpp_cmd += ['-E']  # GCC/Clang preprocessor flag
    
    return cpp_cmd


def get_c_declarations(header=header, usedouble=False):
    ''' return a dense and preprocessed  string of all c declarations implied by aubio.h
    '''
    cpp_output = get_cpp_output(header=header, usedouble=usedouble)
    return filter_cpp_output (cpp_output)


def get_cpp_output(header=header, usedouble=False):
    ''' find and run a C pre-processor on aubio.h '''
    cpp_cmd = get_preprocessor()

    macros = [('AUBIO_UNSTABLE', 1)]
    if usedouble:
        macros += [('HAVE_AUBIO_DOUBLE', 1)]

    if not os.path.isfile(header):
        raise Exception("could not find include file " + header)

    includes = [os.path.dirname(header)]
    
    # Add macro definitions
    for macro_name, macro_value in macros:
        if macro_value is None:
            cpp_cmd += ['-D' + macro_name]
        else:
            cpp_cmd += ['-D{}={}'.format(macro_name, macro_value)]
    
    # Add include paths
    for include_dir in includes:
        cpp_cmd += ['-I' + include_dir]
    
    cpp_cmd += [header]

    print("Running command: {:s}".format(" ".join(cpp_cmd)))
    proc = subprocess.Popen(cpp_cmd,
                            stderr=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            universal_newlines=True)
    assert proc, 'Proc was none'
    cpp_output = proc.stdout.read()
    err_output = proc.stderr.read()
    if err_output:
        print("Warning: preprocessor produced errors or warnings:\n%s" \
                % err_output)
    if not cpp_output:
        raise_msg = "preprocessor output is empty! Running command " \
                + "\"%s\" failed" % " ".join(cpp_cmd)
        if err_output:
            raise_msg += " with stderr: \"%s\"" % err_output
        else:
            raise_msg += " with no stdout or stderr"
        raise Exception(raise_msg)
    if not isinstance(cpp_output, list):
        cpp_output = [l.strip() for l in cpp_output.split('\n')]

    return cpp_output

def filter_cpp_output(cpp_raw_output):
    ''' prepare cpp-output for parsing '''
    cpp_output = filter(lambda y: len(y) > 1, cpp_raw_output)
    cpp_output = list(filter(lambda y: not y.startswith('#'), cpp_output))

    i = 1
    while 1:
        if i >= len(cpp_output):
            break
        if ('{' in cpp_output[i - 1]) and ('}' not in cpp_output[i - 1]) or (';' not in cpp_output[i - 1]):
            cpp_output[i] = cpp_output[i - 1] + ' ' + cpp_output[i]
            cpp_output.pop(i - 1)
        elif ('}' in cpp_output[i]):
            cpp_output[i] = cpp_output[i - 1] + ' ' + cpp_output[i]
            cpp_output.pop(i - 1)
        else:
            i += 1

    # clean pointer notations
    tmp = []
    for l in cpp_output:
        tmp += [l.replace(' *', ' * ')]
    cpp_output = tmp

    return cpp_output


def get_cpp_objects_from_c_declarations(c_declarations, skip_objects=None):
    if skip_objects is None:
        skip_objects = default_skip_objects
    typedefs = filter(lambda y: y.startswith('typedef struct _aubio'), c_declarations)
    cpp_objects = [a.split()[3][:-1] for a in typedefs]
    cpp_objects_filtered = filter(lambda y: not y[6:-2] in skip_objects, cpp_objects)
    return cpp_objects_filtered


def get_all_func_names_from_lib(lib):
    ''' return flat string of all function used in lib
    '''
    res = []
    for _, v in lib.items():
        if isinstance(v, dict):
            res += get_all_func_names_from_lib(v)
        elif isinstance(v, list):
            for elem in v:
                e = elem.split('(')
                if len(e) < 2:
                    continue  # not a function
                fname_part = e[0].strip().split(' ')
                fname = fname_part[-1]
                if fname:
                    res += [fname]
                else:
                    raise NameError('gen_lib : weird function: ' + str(e))

    return res


def generate_lib_from_c_declarations(cpp_objects, c_declarations):
    ''' returns a lib from given cpp_object names

    a lib is a dict grouping functions by family (onset,pitch...)
        each eement is itself a dict of functions grouped by puposes as : 
        struct, new, del, do, get, set and other
    '''
    lib = {}

    for o in cpp_objects:
        shortname = o
        if o[:6] == 'aubio_':
            shortname = o[6:-2]  # without aubio_ prefix and _t suffix

        lib[shortname] = {'struct': [], 'new': [], 'del': [], 'do': [], 'rdo': [], 'get': [], 'set': [], 'other': []}
        lib[shortname]['longname'] = o
        lib[shortname]['shortname'] = shortname

        fullshortname = o[:-2]  # name without _t suffix

        for fn in c_declarations:
            func_name = fn.split('(')[0].strip().split(' ')[-1]
            if func_name.startswith(fullshortname + '_') or func_name.endswith(fullshortname):
                # print "found", shortname, "in", fn
                if 'typedef struct ' in fn:
                    lib[shortname]['struct'].append(fn)
                elif '_do' in fn:
                    lib[shortname]['do'].append(fn)
                elif '_rdo' in fn:
                    lib[shortname]['rdo'].append(fn)
                elif 'new_' in fn:
                    lib[shortname]['new'].append(fn)
                elif 'del_' in fn:
                    lib[shortname]['del'].append(fn)
                elif '_get_' in fn:
                    lib[shortname]['get'].append(fn)
                elif '_set_' in fn:
                    lib[shortname]['set'].append(fn)
                else:
                    # print "no idea what to do about", fn
                    lib[shortname]['other'].append(fn)
    return lib


def print_c_declarations_results(lib, c_declarations):
    for fn in c_declarations:
        found = 0
        for o in lib:
            for family in lib[o]:
                if fn in lib[o][family]:
                    found = 1
        if found == 0:
            print("missing", fn)

    for o in lib:
        for family in lib[o]:
            if type(lib[o][family]) == str:
                print("{:15s} {:10s} {:s}".format(o, family, lib[o][family]))
            elif len(lib[o][family]) == 1:
                print("{:15s} {:10s} {:s}".format(o, family, lib[o][family][0]))
            else:
                print("{:15s} {:10s} {:s}".format(o, family, lib[o][family]))


def generate_external(header=header, output_path=output_path, usedouble=False, overwrite=True):
    if not os.path.isdir(output_path):
        os.mkdir(output_path)
    elif not overwrite:
        return sorted(glob.glob(os.path.join(output_path, '*.c')))

    c_declarations = get_c_declarations(header, usedouble=usedouble)
    cpp_objects = get_cpp_objects_from_c_declarations(c_declarations)

    lib = generate_lib_from_c_declarations(cpp_objects, c_declarations)
    # print_c_declarations_results(lib, c_declarations)

    sources_list = []
    for o in lib:
        out = source_header
        mapped = MappedObject(lib[o], usedouble=usedouble)
        out += mapped.gen_code()
        output_file = os.path.join(output_path, 'gen-%s.c' % o)
        with open(output_file, 'w') as f:
            f.write(out)
            print("wrote %s" % output_file)
            sources_list.append(output_file)

    out = source_header
    out += "#include \"aubio-generated.h\""
    check_types = "\n     ||  ".join(["PyType_Ready(&Py_%sType) < 0" % o for o in lib])
    out += """

int generated_types_ready (void)
{{
  return ({pycheck_types});
}}
""".format(pycheck_types=check_types)

    add_types = "".join(["""
  Py_INCREF (&Py_{name}Type);
  PyModule_AddObject(m, "{name}", (PyObject *) & Py_{name}Type);""".format(name=o) for o in lib])
    out += """

void add_generated_objects ( PyObject *m )
{{
{add_types}
}}
""".format(add_types=add_types)

    output_file = os.path.join(output_path, 'aubio-generated.c')
    with open(output_file, 'w') as f:
        f.write(out)
        print("wrote %s" % output_file)
        sources_list.append(output_file)

    objlist = "".join(["extern PyTypeObject Py_%sType;\n" % p for p in lib])
    out = """// generated list of objects created with gen_external.py

#include <Python.h>
"""
    if usedouble:
        out += """
#ifndef HAVE_AUBIO_DOUBLE
#define HAVE_AUBIO_DOUBLE 1
#endif
"""
    out += """
{objlist}
int generated_objects ( void );
void add_generated_objects( PyObject *m );
""".format(objlist=objlist)

    output_file = os.path.join(output_path, 'aubio-generated.h')
    with open(output_file, 'w') as f:
        f.write(out)
        print("wrote %s" % output_file)
        # no need to add header to list of sources

    return sorted(sources_list)

if __name__ == '__main__':
    if len(sys.argv) > 1:
        header = sys.argv[1]
    if len(sys.argv) > 2:
        output_path = sys.argv[2]
    generate_external(header, output_path)



#! /usr/bin/env python

import sys
from aubio import source, pitch

if len(sys.argv) < 2:
    print("Usage: %s <filename> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]

downsample = 1
samplerate = 44100 // downsample
if len( sys.argv ) > 2: samplerate = int(sys.argv[2])

win_s = 4096 // downsample # fft size
hop_s = 512  // downsample # hop size

s = source(filename, samplerate, hop_s)
samplerate = s.samplerate

tolerance = 0.8

pitch_o = pitch("yin", win_s, hop_s, samplerate)
pitch_o.set_unit("midi")
pitch_o.set_tolerance(tolerance)

pitches = []
confidences = []

# total number of frames read
total_frames = 0
while True:
    samples, read = s()
    pitch = pitch_o(samples)[0]
    #pitch = int(round(pitch))
    confidence = pitch_o.get_confidence()
    #if confidence < 0.8: pitch = 0.
    print("%f %f %f" % (total_frames / float(samplerate), pitch, confidence))
    pitches += [pitch]
    confidences += [confidence]
    total_frames += read
    if read < hop_s: break

if 0: sys.exit(0)

#print pitches
import os.path
from numpy import array, ma
import matplotlib.pyplot as plt
from demo_waveform_plot import get_waveform_plot, set_xlabels_sample2time

skip = 1

pitches = array(pitches[skip:])
confidences = array(confidences[skip:])
times = [t * hop_s for t in range(len(pitches))]

fig = plt.figure()

ax1 = fig.add_subplot(311)
ax1 = get_waveform_plot(filename, samplerate = samplerate, block_size = hop_s, ax = ax1)
plt.setp(ax1.get_xticklabels(), visible = False)
ax1.set_xlabel('')

def array_from_text_file(filename, dtype = 'float'):
    filename = os.path.join(os.path.dirname(__file__), filename)
    return array([line.split() for line in open(filename).readlines()],
        dtype = dtype)

ax2 = fig.add_subplot(312, sharex = ax1)
ground_truth = os.path.splitext(filename)[0] + '.f0.Corrected'
if os.path.isfile(ground_truth):
    ground_truth = array_from_text_file(ground_truth)
    true_freqs = ground_truth[:,2]
    true_freqs = ma.masked_where(true_freqs < 2, true_freqs)
    true_times = float(samplerate) * ground_truth[:,0]
    ax2.plot(true_times, true_freqs, 'r')
    ax2.axis( ymin = 0.9 * true_freqs.min(), ymax = 1.1 * true_freqs.max() )
# plot raw pitches
ax2.plot(times, pitches, '.g')
# plot cleaned up pitches
cleaned_pitches = pitches
#cleaned_pitches = ma.masked_where(cleaned_pitches < 0, cleaned_pitches)
#cleaned_pitches = ma.masked_where(cleaned_pitches > 120, cleaned_pitches)
cleaned_pitches = ma.masked_where(confidences < tolerance, cleaned_pitches)
ax2.plot(times, cleaned_pitches, '.-')
#ax2.axis( ymin = 0.9 * cleaned_pitches.min(), ymax = 1.1 * cleaned_pitches.max() )
#ax2.axis( ymin = 55, ymax = 70 )
plt.setp(ax2.get_xticklabels(), visible = False)
ax2.set_ylabel('f0 (midi)')

# plot confidence
ax3 = fig.add_subplot(313, sharex = ax1)
# plot the confidence
ax3.plot(times, confidences)
# draw a line at tolerance
ax3.plot(times, [tolerance]*len(confidences))
ax3.axis( xmin = times[0], xmax = times[-1])
ax3.set_ylabel('confidence')
set_xlabels_sample2time(ax3, times[-1], samplerate)
plt.show()
#plt.savefig(os.path.basename(filename) + '.svg')



#! /usr/bin/env python

from numpy.testing import TestCase, assert_equal, assert_array_less
from _tools import parametrize, skipTest
from aubio import fvec, cvec, pvoc, float_type
import numpy as np

if float_type == 'float32':
    max_sq_error = 1.e-12
else:
    max_sq_error = 1.e-29

def create_sine(hop_s, freq, samplerate):
    t = np.arange(hop_s).astype(float_type)
    return np.sin( 2. * np.pi * freq * t / float(samplerate))

def create_noise(hop_s):
    return np.random.rand(hop_s).astype(float_type) * 2. - 1.

class Test_aubio_pvoc_test_case(object):
    """ pvoc object test case """

    def test_members_automatic_sizes_default(self):
        """ check object creation with default parameters """
        f = pvoc()
        assert_equal ([f.win_s, f.hop_s], [1024, 512])

    def test_members_unnamed_params(self):
        """ check object creation with unnamed parameters """
        f = pvoc(2048, 128)
        assert_equal ([f.win_s, f.hop_s], [2048, 128])

    def test_members_named_params(self):
        """ check object creation with named parameters """
        f = pvoc(hop_s = 128, win_s = 2048)
        assert_equal ([f.win_s, f.hop_s], [2048, 128])

    def test_zeros(self):
        """ check the resynthesis of zeros gives zeros """
        win_s, hop_s = 1024, 256
        f = pvoc (win_s, hop_s)
        t = fvec (hop_s)
        for _ in range( int ( 4 * win_s / hop_s ) ):
            s = f(t)
            r = f.rdo(s)
            assert_equal ( t, 0.)
            assert_equal ( s.norm, 0.)
            try:
                assert_equal ( s.phas, 0 )
            except AssertionError:
                assert_equal (s.phas[s.phas > 0], +np.pi)
                assert_equal (s.phas[s.phas < 0], -np.pi)
                assert_equal (np.abs(s.phas[np.abs(s.phas) != np.pi]), 0)
                skipTest('pvoc(fvec(%d)).phas != +0, ' % win_s \
                        + 'This is expected when using fftw3 on powerpc.')
            assert_equal ( r, 0.)

    def test_no_overlap(self):
        win_s, hop_s = 1024, 1024
        f = pvoc (win_s, hop_s)
        t = fvec (hop_s)
        for _ in range(4):
            s = f(t)
            r = f.rdo(s)
            assert_equal ( t, 0.)

    resynth_noise_args = "hop_s, ratio"
    resynth_noise_values = [
            ( 256, 8),
            ( 256, 4),
            ( 256, 2),
            ( 512, 8),
            ( 512, 4),
            ( 512, 2),
            #( 129, 2),
            #( 320, 4),
            #(  13, 8),
            (1024, 8),
            (1024, 4),
            (1024, 2),
            (2048, 8),
            (2048, 4),
            (2048, 2),
            (4096, 8),
            (4096, 4),
            (4096, 2),
            (8192, 8),
            (8192, 4),
            (8192, 2),
            ]

    @parametrize(resynth_noise_args, resynth_noise_values)
    def test_resynth_steps_noise(self, hop_s, ratio):
        """ check the resynthesis of a random signal is correct """
        sigin = create_noise(hop_s)
        self.reconstruction(sigin, hop_s, ratio)

    resynth_sine_args = "samplerate, hop_s, ratio, freq"
    resynth_sine_values = [
            (44100,  256, 8,   441),
            (44100,  256, 4,  1203),
            (44100,  256, 2,  3045),
            (44100,  512, 8,   445),
            (44100,  512, 4,   445),
            (44100,  512, 2,   445),
            (44100, 1024, 8,   445),
            (44100, 1024, 4,   445),
            (44100, 1024, 2,   445),
            ( 8000, 1024, 2,   445),
            (22050, 1024, 2,   445),
            (22050,  256, 8,   445),
            (96000, 1024, 8, 47000),
            (96000, 1024, 8,    20),
            ]

    @parametrize(resynth_sine_args, resynth_sine_values)
    def test_resynth_steps_sine(self, samplerate, hop_s, ratio, freq):
        """ check the resynthesis of a sine is correct """
        sigin = create_sine(hop_s, freq, samplerate)
        self.reconstruction(sigin, hop_s, ratio)

    def reconstruction(self, sigin, hop_s, ratio):
        buf_s = hop_s * ratio
        f = pvoc(buf_s, hop_s)
        zeros = fvec(hop_s)
        r2 = f.rdo( f(sigin) )
        for _ in range(1, ratio):
            r2 = f.rdo( f(zeros) )
        # compute square errors
        sq_error = (r2 - sigin)**2
        # make sure all square errors are less than desired precision
        assert_array_less(sq_error, max_sq_error)

class aubio_pvoc_strange_params(TestCase):

    def test_win_size_short(self):
        with self.assertRaises(RuntimeError):
            pvoc(1, 1)

    def test_hop_size_long(self):
        with self.assertRaises(RuntimeError):
            pvoc(1024, 1025)

    def test_large_input_timegrain(self):
        win_s = 1024
        f = pvoc(win_s)
        t = fvec(win_s + 1)
        with self.assertRaises(ValueError):
            f(t)

    def test_small_input_timegrain(self):
        win_s = 1024
        f = pvoc(win_s)
        t = fvec(1)
        with self.assertRaises(ValueError):
            f(t)

    def test_large_input_fftgrain(self):
        win_s = 1024
        f = pvoc(win_s)
        s = cvec(win_s + 5)
        with self.assertRaises(ValueError):
            f.rdo(s)

    def test_small_input_fftgrain(self):
        win_s = 1024
        f = pvoc(win_s)
        s = cvec(16)
        with self.assertRaises(ValueError):
            f.rdo(s)

class aubio_pvoc_wrong_params(TestCase):

    def test_wrong_buf_size(self):
        win_s = -1
        with self.assertRaises(ValueError):
            pvoc(win_s)

    def test_buf_size_too_small(self):
        win_s = 1
        with self.assertRaises(RuntimeError):
            pvoc(win_s)

    def test_hop_size_negative(self):
        win_s = 512
        hop_s = -2
        with self.assertRaises(ValueError):
            pvoc(win_s, hop_s)

    def test_hop_size_too_small(self):
        win_s = 1
        hop_s = 1
        with self.assertRaises(RuntimeError):
            pvoc(win_s, hop_s)

    def test_buf_size_not_power_of_two(self):
        win_s = 320
        hop_s = win_s // 2
        try:
            with self.assertRaises(RuntimeError):
                pvoc(win_s, hop_s)
        except AssertionError:
            # when compiled with fftw3, aubio supports non power of two fft sizes
            self.skipTest('creating aubio.pvoc with size %d did not fail' % win_s)

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

import numpy as np
from numpy.testing import TestCase
from numpy.testing import assert_equal, assert_almost_equal
from aubio import window, level_lin, db_spl, silence_detection, level_detection
from aubio import fvec, float_type

class aubio_window(TestCase):

    def test_accept_name_and_size(self):
        window("default", 1024)

    def test_fail_name_not_string(self):
        with self.assertRaises(TypeError):
            window(10, 1024)

    def test_fail_size_not_int(self):
        with self.assertRaises(TypeError):
            window("default", "default")

    def test_compute_hanning_1024(self):
        size = 1024
        aubio_window = window("hanning", size)
        numpy_window = .5 - .5 * np.cos(2. * np.pi * np.arange(size) / size)
        assert_almost_equal(aubio_window, numpy_window)

class aubio_level_lin(TestCase):
    def test_accept_fvec(self):
        level_lin(fvec(1024))

    def test_fail_not_fvec(self):
        with self.assertRaises(ValueError):
            level_lin("default")

    def test_zeros_is_zeros(self):
        assert_equal(level_lin(fvec(1024)), 0.)

    def test_minus_ones_is_one(self):
        assert_equal(level_lin(-np.ones(1024, dtype = float_type)), 1.)

class aubio_db_spl(TestCase):
    def test_accept_fvec(self):
        db_spl(fvec(1024))

    def test_fail_not_fvec(self):
        with self.assertRaises(ValueError):
            db_spl("default")

    def test_zeros_is_inf(self):
        assert np.isinf(db_spl(fvec(1024)))

    def test_minus_ones_is_zero(self):
        assert_equal(db_spl(-np.ones(1024, dtype = float_type)), 0.)

class aubio_silence_detection(TestCase):
    def test_accept_fvec(self):
        silence_detection(fvec(1024), -70.)

    def test_fail_not_fvec(self):
        with self.assertRaises(ValueError):
            silence_detection("default", -70)

    def test_zeros_is_one(self):
        assert silence_detection(fvec(1024), -70) == 1

    def test_minus_ones_is_zero(self):
        from numpy import ones
        assert silence_detection(ones(1024, dtype = float_type), -70) == 0

class aubio_level_detection(TestCase):
    def test_accept_fvec(self):
        level_detection(fvec(1024), -70.)

    def test_fail_not_fvec(self):
        with self.assertRaises(ValueError):
            level_detection("default", -70)

    def test_zeros_is_one(self):
        assert level_detection(fvec(1024), -70) == 1

    def test_minus_ones_is_zero(self):
        from numpy import ones
        assert level_detection(ones(1024, dtype = float_type), -70) == 0

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

from unittest import main
from numpy.testing import TestCase, assert_equal, assert_almost_equal
import aubio

class aubio_tempo_default(TestCase):

    def test_members(self):
        o = aubio.tempo()
        assert_equal ([o.buf_size, o.hop_size, o.method, o.samplerate],
            [1024,512,'default',44100])

class aubio_tempo_params(TestCase):

    samplerate = 44100

    def setUp(self):
        self.o = aubio.tempo(samplerate = self.samplerate)

    def test_get_delay(self):
        self.assertEqual(self.o.get_delay(), 0)

    def test_set_delay(self):
        val = 256
        self.o.set_delay(val)
        assert_equal (self.o.get_delay(), val)

    def test_get_delay_s(self):
        self.assertEqual(self.o.get_delay_s(), 0.)

    def test_set_delay_s(self):
        val = .05
        self.o.set_delay_s(val)
        assert_almost_equal (self.o.get_delay_s(), val)

    def test_get_delay_ms(self):
        self.assertEqual(self.o.get_delay_ms(), 0.)

    def test_set_delay_ms(self):
        val = 50.
        self.o.set_delay_ms(val)
        assert_almost_equal (self.o.get_delay_ms(), val)

    def test_get_threshold(self):
        assert_almost_equal(self.o.get_threshold(), 0.3)

    def test_set_threshold(self):
        val = .1
        self.o.set_threshold(val)
        assert_almost_equal (self.o.get_threshold(), val)

    def test_get_silence(self):
        self.assertEqual(self.o.get_silence(), -90.)

    def test_set_silence(self):
        val = -50.
        self.o.set_silence(val)
        assert_almost_equal (self.o.get_silence(), val)

    def test_get_last(self):
        self.assertEqual(self.o.get_last(), 0.)

    def test_get_last_s(self):
        self.assertEqual(self.o.get_last_s(), 0.)

    def test_get_last_ms(self):
        self.assertEqual(self.o.get_last_ms(), 0.)

    def test_get_period(self):
        self.assertEqual(self.o.get_period(), 0.)

    def test_get_period_s(self):
        self.assertEqual(self.o.get_period_s(), 0.)

    def test_get_last_tatum(self):
        self.assertEqual(self.o.get_last_tatum(), 0.)

    def test_set_tatum_signature(self):
        self.o.set_tatum_signature(8)
        self.o.set_tatum_signature(64)
        self.o.set_tatum_signature(1)

    def test_set_wrong_tatum_signature(self):
        with self.assertRaises(ValueError):
            self.o.set_tatum_signature(101)
        with self.assertRaises(ValueError):
            self.o.set_tatum_signature(0)

if __name__ == '__main__':
    main()



#! /usr/bin/env python

import sys
from aubio import source, onset

win_s = 512                 # fft size
hop_s = win_s // 2          # hop size

if len(sys.argv) < 2:
    print("Usage: %s <filename> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]

samplerate = 0
if len( sys.argv ) > 2: samplerate = int(sys.argv[2])

s = source(filename, samplerate, hop_s)
samplerate = s.samplerate

o = onset("default", win_s, hop_s, samplerate)

# list of onsets, in samples
onsets = []

# total number of frames read
total_frames = 0
while True:
    samples, read = s()
    if o(samples):
        print("%f" % o.get_last_s())
        onsets.append(o.get_last())
    total_frames += read
    if read < hop_s: break
#print len(onsets)



#! /usr/bin/env python

from unittest import main
from numpy.testing import TestCase
from numpy.testing import assert_equal, assert_almost_equal
from _tools import assert_warns
from utils import is32bit
import numpy as np
import aubio

from aubio import hztomel, meltohz
from aubio import hztomel_htk, meltohz_htk

class aubio_hztomel_test_case(TestCase):

    def test_hztomel(self):
        assert_equal(hztomel(0.), 0.)
        assert_almost_equal(hztomel(400. / 3.), 2., decimal=5)
        assert_almost_equal(hztomel(1000. / 3), 5.)
        # on 32bit, some of these tests fails unless compiling with -ffloat-store
        try:
            assert_equal(hztomel(200.), 3.)
        except AssertionError:
            if not is32bit(): raise
            assert_almost_equal(hztomel(200.), 3., decimal=5)
        assert_almost_equal(hztomel(1000.), 15)
        assert_almost_equal(hztomel(6400), 42, decimal=5)
        assert_almost_equal(hztomel(40960), 69, decimal=5)

        for m in np.linspace(0, 1000, 100):
            assert_almost_equal(hztomel(meltohz(m)) - m, 0, decimal=3)

    def test_meltohz(self):
        assert_equal(meltohz(0.), 0.)
        assert_almost_equal(meltohz(2), 400. / 3., decimal=4)
        try:
            assert_equal(meltohz(3.), 200.)
        except AssertionError:
            if not is32bit(): raise
            assert_almost_equal(meltohz(3.), 200., decimal=5)
        assert_almost_equal(meltohz(5), 1000. / 3., decimal=4)
        assert_almost_equal(meltohz(15), 1000., decimal=4)
        assert_almost_equal(meltohz(42), 6400., decimal=2)
        assert_almost_equal(meltohz(69), 40960., decimal=1)

        for f in np.linspace(0, 20000, 1000):
            assert_almost_equal(meltohz(hztomel(f)) - f, 0, decimal=1)

    def test_meltohz_negative(self):
        with assert_warns(UserWarning):
            assert_equal(meltohz(-1), 0)

    def test_hztomel_negative(self):
        with assert_warns(UserWarning):
            assert_equal(hztomel(-1), 0)


class aubio_hztomel_htk_test_case(TestCase):

    def test_meltohz(self):
        assert_equal(meltohz(0, htk=True), 0)
        assert_almost_equal(meltohz(2595, htk=True), 6300., decimal=1)

    def test_hztomel(self):
        assert_equal(hztomel(0, htk=True), 0)
        assert_almost_equal(hztomel(3428.7, htk=True), 2000., decimal=1)
        assert_almost_equal(hztomel(6300, htk=True), 2595., decimal=1)

    def test_meltohz_negative(self):
        with assert_warns(UserWarning):
            assert_equal(meltohz(-1, htk=True), 0)
        assert_almost_equal(meltohz(2000, htk=True), 3428.7, decimal=1)
        assert_almost_equal(meltohz(1000, htk=True), 1000., decimal=1)

    def test_hztomel_negative(self):
        with assert_warns(UserWarning):
            assert_equal(meltohz(-1, htk=True), 0)
        with assert_warns(UserWarning):
            assert_equal(hztomel(-1, htk=True), 0)
        assert_almost_equal(hztomel(1000, htk=True), 1000., decimal=1)

    def test_hztomel_htk(self):
        for f in np.linspace(0, 20000, 1000):
            assert_almost_equal(meltohz_htk(hztomel_htk(f)) - f, 0, decimal=1)
        for f in np.linspace(0, 20000, 1000):
            assert_almost_equal(hztomel_htk(meltohz_htk(f)) - f, 0, decimal=1)


class aubio_hztomel_wrong_values(TestCase):
    """ more tests to cover all branches """

    def test_hztomel_wrong_values(self):
        with self.assertRaises(TypeError):
            hztomel('s')

    def test_meltohz_wrong_values(self):
        with self.assertRaises(TypeError):
            meltohz(bytes('ad'))

    def test_meltohz_no_arg(self):
        with self.assertRaises(TypeError):
            meltohz()

    def test_meltohz_htk_no_arg(self):
        with self.assertRaises(TypeError):
            meltohz_htk()

    def test_hztomel_htk_wrong_values(self):
        with self.assertRaises(TypeError):
            hztomel_htk('0')

    def test_hztomel_htk_false(self):
        assert hztomel(120, htk=False) == hztomel(120)

    def test_meltohz_htk_false(self):
        assert meltohz(12, htk=False) == meltohz(12)


if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

import aubio.cut
from numpy.testing import TestCase

class aubio_cut(TestCase):

    def setUp(self):
        self.a_parser = aubio.cut.aubio_cut_parser()

    def test_default_creation(self):
        assert self.a_parser.parse_args(['-v']).verbose

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

def play_source(source_path):
    """Play an audio file using pysoundcard."""

    from aubio import source
    from pysoundcard import Stream
    
    hop_size = 256
    f = source(source_path, hop_size = hop_size)
    samplerate = f.samplerate

    s = Stream(samplerate = samplerate, blocksize = hop_size)
    s.start()
    read = 0
    while 1:
        vec, read = f()
        s.write(vec)
        if read < hop_size: break
    s.stop()

if __name__ == '__main__':
    import sys
    play_source(sys.argv[1])



#! /usr/bin/env python

from numpy.testing import TestCase, assert_equal, assert_almost_equal
from aubio import onset, fvec

class aubio_onset_default(TestCase):

    def test_members(self):
        o = onset()
        assert_equal ([o.buf_size, o.hop_size, o.method, o.samplerate],
            [1024,512,'default',44100])

class aubio_onset_params(TestCase):

    samplerate = 44100

    def setUp(self):
        self.o = onset(samplerate = self.samplerate)

    def test_get_delay(self):
        self.assertGreater(self.o.get_delay(), 0)

    def test_get_delay_s(self):
        self.assertGreater(self.o.get_delay_s(), 0.)

    def test_get_delay_ms(self):
        self.assertGreater(self.o.get_delay_ms(), 0.)

    def test_get_minioi(self):
        self.assertGreater(self.o.get_minioi(), 0)

    def test_get_minioi_s(self):
        self.assertGreater(self.o.get_minioi_s(), 0.)

    def test_get_minioi_ms(self):
        self.assertGreater(self.o.get_minioi_ms(), 0.)

    def test_get_threshold(self):
        self.assertGreater(self.o.get_threshold(), 0.)

    def test_set_delay(self):
        val = 256
        self.o.set_delay(val)
        assert_equal (self.o.get_delay(), val)

    def test_set_delay_s(self):
        val = .05
        self.o.set_delay_s(val)
        assert_almost_equal (self.o.get_delay_s(), val)

    def test_set_delay_ms(self):
        val = 50.
        self.o.set_delay_ms(val)
        assert_almost_equal (self.o.get_delay_ms(), val)

    def test_set_minioi(self):
        val = 200
        self.o.set_minioi(val)
        assert_equal (self.o.get_minioi(), val)

    def test_set_minioi_s(self):
        val = 0.04
        self.o.set_minioi_s(val)
        assert_almost_equal (self.o.get_minioi_s(), val)

    def test_set_minioi_ms(self):
        val = 40.
        self.o.set_minioi_ms(val)
        assert_almost_equal (self.o.get_minioi_ms(), val)

    def test_set_threshold(self):
        val = 0.2
        self.o.set_threshold(val)
        assert_almost_equal (self.o.get_threshold(), val)

class aubio_onset_96000(aubio_onset_params):
    samplerate = 96000

class aubio_onset_32000(aubio_onset_params):
    samplerate = 32000

class aubio_onset_8000(aubio_onset_params):
    samplerate = 8000

class aubio_onset_coverate(TestCase):
    # extra tests to execute the C routines and improve coverage

    def test_all_methods(self):
        for method in ['default', 'energy', 'hfc', 'complexdomain', 'complex',
                'phase', 'wphase', 'mkl', 'kl', 'specflux', 'specdiff',
                'old_default']:
            o = onset(method=method, buf_size=512, hop_size=256)
            o(fvec(256))

    def test_get_methods(self):
        o = onset(method='default', buf_size=512, hop_size=256)

        assert o.get_silence() == -70
        o.set_silence(-20)
        assert_almost_equal(o.get_silence(), -20)

        assert o.get_compression() == 1
        o.set_compression(.99)
        assert_almost_equal(o.get_compression(), .99)

        assert o.get_awhitening() == 0
        o.set_awhitening(1)
        assert o.get_awhitening() == 1

        o.get_last()
        o.get_last_ms()
        o.get_last_s()
        o.get_descriptor()
        o.get_thresholded_descriptor()


if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python


import numpy as np
from numpy.testing import TestCase, assert_almost_equal
import aubio

precomputed_arange = [ 9.89949512, -6.44232273,  0., -0.67345482, 0.,
        -0.20090288, 0., -0.05070186]

precomputed_some_ones = [ 4.28539848,  0.2469689,  -0.14625292, -0.58121818,
        -0.83483052, -0.75921834, -0.35168475,  0.24087936,
        0.78539824, 1.06532764,  0.97632152,  0.57164496, 0.03688532,
        -0.39446154, -0.54619485, -0.37771079]

class aubio_dct(TestCase):

    def test_init(self):
        """ test that aubio.dct() is created with expected size """
        a_dct = aubio.dct()
        self.assertEqual(a_dct.size, 1024)

    def test_arange(self):
        """ test that dct(arange(8)) is computed correctly

        >>> from scipy.fftpack import dct
        >>> a_in = np.arange(8).astype(aubio.float_type)
        >>> precomputed = dct(a_in, norm='ortho')
        """
        N = len(precomputed_arange)
        a_dct = aubio.dct(8)
        a_in = np.arange(8).astype(aubio.float_type)
        a_expected = aubio.fvec(precomputed_arange)
        assert_almost_equal(a_dct(a_in), a_expected, decimal=5)

    def test_some_ones(self):
        """ test that dct(somevector) is computed correctly """
        a_dct = aubio.dct(16)
        a_in = np.ones(16).astype(aubio.float_type)
        a_in[1] = 0
        a_in[3] = np.pi
        a_expected = aubio.fvec(precomputed_some_ones)
        assert_almost_equal(a_dct(a_in), a_expected, decimal=6)

    def test_reconstruction(self):
        """ test that some_ones vector can be recontructed """
        a_dct = aubio.dct(16)
        a_in = np.ones(16).astype(aubio.float_type)
        a_in[1] = 0
        a_in[3] = np.pi
        a_dct_in = a_dct(a_in)
        a_dct_reconstructed = a_dct.rdo(a_dct_in)
        assert_almost_equal(a_dct_reconstructed, a_in, decimal=6)

    def test_negative_size(self):
        """ test that creation fails with a negative size """
        with self.assertRaises(ValueError):
            aubio.dct(-1)

    def test_wrong_size(self):
        """ test that creation fails with a non power-of-two size """
        # supports for non 2** fft sizes only when compiled with fftw3
        size = 13
        try:
            with self.assertRaises(RuntimeError):
                aubio.dct(size)
        except AssertionError:
            self.skipTest('creating aubio.dct with size %d did not fail' % size)

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""aubio command line tool

This file was written by Paul Brossier <piem@aubio.org> and is released under
the GNU/GPL v3.

Note: this script is mostly about parsing command line arguments. For more
readable code examples, check out the `python/demos` folder."""

import sys
import argparse
import warnings
import aubio

def aubio_parser():
    epilog = 'use "%(prog)s <command> --help" for more info about each command'
    parser = argparse.ArgumentParser(epilog=epilog)
    parser.add_argument('-V', '--version', help="show version",
            action="store_true", dest="show_version")

    subparsers = parser.add_subparsers(title='commands', dest='command',
            parser_class= AubioArgumentParser,
            metavar="")

    parser_add_subcommand_help(subparsers)

    parser_add_subcommand_onset(subparsers)
    parser_add_subcommand_pitch(subparsers)
    parser_add_subcommand_beat(subparsers)
    parser_add_subcommand_tempo(subparsers)
    parser_add_subcommand_notes(subparsers)
    parser_add_subcommand_mfcc(subparsers)
    parser_add_subcommand_melbands(subparsers)
    parser_add_subcommand_quiet(subparsers)
    parser_add_subcommand_cut(subparsers)

    return parser

def parser_add_subcommand_help(subparsers):
    # global help subcommand
    subparsers.add_parser('help',
            help='show help message',
            formatter_class = argparse.ArgumentDefaultsHelpFormatter)

def parser_add_subcommand_onset(subparsers):
    # onset subcommand
    subparser = subparsers.add_parser('onset',
            help='estimate time of onsets (beginning of sound event)',
            formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    subparser.add_input()
    subparser.add_buf_hop_size()
    helpstr = "onset novelty function"
    helpstr += " <default|energy|hfc|complex|phase|specdiff|kl|mkl|specflux>"
    subparser.add_method(helpstr=helpstr)
    subparser.add_threshold()
    subparser.add_silence()
    subparser.add_minioi()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_onset)

def parser_add_subcommand_pitch(subparsers):
    # pitch subcommand
    subparser = subparsers.add_parser('pitch',
            help='estimate fundamental frequency (monophonic)')
    subparser.add_input()
    subparser.add_buf_hop_size(buf_size=2048)
    helpstr = "pitch detection method <default|yinfft|yin|mcomb|fcomb|schmitt>"
    subparser.add_method(helpstr=helpstr)
    subparser.add_threshold()
    subparser.add_pitch_unit()
    subparser.add_silence()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_pitch)

def parser_add_subcommand_beat(subparsers):
    # beat subcommand
    subparser = subparsers.add_parser('beat',
            help='estimate location of beats')
    subparser.add_input()
    subparser.add_buf_hop_size(buf_size=1024, hop_size=512)
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_beat)

def parser_add_subcommand_tempo(subparsers):
    # tempo subcommand
    subparser = subparsers.add_parser('tempo',
            help='estimate overall tempo in bpm')
    subparser.add_input()
    subparser.add_buf_hop_size(buf_size=1024, hop_size=512)
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_tempo)

def parser_add_subcommand_notes(subparsers):
    # notes subcommand
    subparser = subparsers.add_parser('notes',
            help='estimate midi-like notes (monophonic)')
    subparser.add_input()
    subparser.add_buf_hop_size()
    subparser.add_silence()
    subparser.add_release_drop()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_notes)

def parser_add_subcommand_mfcc(subparsers):
    # mfcc subcommand
    subparser = subparsers.add_parser('mfcc',
            help='extract Mel-Frequency Cepstrum Coefficients')
    subparser.add_input()
    subparser.add_buf_hop_size()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_mfcc)

def parser_add_subcommand_melbands(subparsers):
    # melbands subcommand
    subparser = subparsers.add_parser('melbands',
            help='extract energies in Mel-frequency bands')
    subparser.add_input()
    subparser.add_buf_hop_size()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_melbands)

def parser_add_subcommand_quiet(subparsers):
    # quiet subcommand
    subparser = subparsers.add_parser('quiet',
            help='extract timestamps of quiet and loud regions')
    subparser.add_input()
    subparser.add_hop_size()
    subparser.add_silence()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_quiet)

def parser_add_subcommand_cut(subparsers):
    # cut subcommand
    subparser = subparsers.add_parser('cut',
            help='slice at timestamps')
    subparser.add_input()
    helpstr = "onset novelty function"
    helpstr += " <default|energy|hfc|complex|phase|specdiff|kl|mkl|specflux>"
    subparser.add_method(helpstr=helpstr)
    subparser.add_buf_hop_size()
    subparser.add_silence()
    subparser.add_threshold(default=0.3)
    subparser.add_minioi()
    subparser.add_slicer_options()
    subparser.add_time_format()
    subparser.add_verbose_help()
    subparser.set_defaults(process=process_cut)

class AubioArgumentParser(argparse.ArgumentParser):

    def add_input(self):
        self.add_argument("source_uri", default=None, nargs='?',
                help="input sound file to analyse", metavar = "<source_uri>")
        self.add_argument("-i", "--input", dest = "source_uri2",
                help="input sound file to analyse", metavar = "<source_uri>")
        self.add_argument("-r", "--samplerate",
                metavar = "<freq>", type=int,
                action="store", dest="samplerate", default=0,
                help="samplerate at which the file should be represented")

    def add_verbose_help(self):
        self.add_argument("-v", "--verbose",
                action="count", dest="verbose", default=1,
                help="make lots of noise [default]")
        self.add_argument("-q", "--quiet",
                action="store_const", dest="verbose", const=0,
                help="be quiet")

    def add_buf_hop_size(self, buf_size=512, hop_size=256):
        self.add_buf_size(buf_size=buf_size)
        self.add_hop_size(hop_size=hop_size)

    def add_buf_size(self, buf_size=512):
        self.add_argument("-B", "--bufsize",
                action="store", dest="buf_size", default=buf_size,
                metavar = "<size>", type=int,
                help="buffer size [default=%d]" % buf_size)

    def add_hop_size(self, hop_size=256):
        self.add_argument("-H", "--hopsize",
                metavar = "<size>", type=int,
                action="store", dest="hop_size", default=hop_size,
                help="overlap size [default=%d]" % hop_size)

    def add_method(self, method='default', helpstr='method'):
        self.add_argument("-m", "--method",
                metavar = "<method>", type=str,
                action="store", dest="method", default=method,
                help="%s [default=%s]" % (helpstr, method))

    def add_threshold(self, default=None):
        self.add_argument("-t", "--threshold",
                metavar = "<threshold>", type=float,
                action="store", dest="threshold", default=default,
                help="threshold [default=%s]" % default)

    def add_silence(self):
        self.add_argument("-s", "--silence",
                metavar = "<value>", type=float,
                action="store", dest="silence", default=-70,
                help="silence threshold")

    def add_release_drop(self):
        self.add_argument("-d", "--release-drop",
                metavar = "<value>", type=float,
                action="store", dest="release_drop", default=10,
                help="release drop threshold")

    def add_minioi(self, default="12ms"):
        self.add_argument("-M", "--minioi",
                metavar = "<value>", type=str,
                action="store", dest="minioi", default=default,
                help="minimum Inter-Onset Interval [default=%s]" % default)

    def add_pitch_unit(self, default="Hz"):
        help_str = "frequency unit, should be one of Hz, midi, bin, cent"
        help_str += " [default=%s]" % default
        self.add_argument("-u", "--pitch-unit",
                metavar = "<value>", type=str,
                action="store", dest="pitch_unit", default=default,
                help=help_str)

    def add_time_format(self):
        helpstr = "select time values output format (samples, ms, seconds)"
        helpstr += " [default=seconds]"
        self.add_argument("-T", "--time-format",
                 metavar='format',
                 dest="time_format",
                 default=None,
                 help=helpstr)

    def add_slicer_options(self):
        self.add_argument("-o", "--output", type = str,
                metavar = "<outputdir>",
                action="store", dest="output_directory", default=None,
                help="specify path where slices of the original file should"
                " be created")
        self.add_argument("--cut-until-nsamples", type = int,
                metavar = "<samples>",
                action = "store", dest = "cut_until_nsamples", default = None,
                help="how many extra samples should be added at the end of"
                " each slice")
        self.add_argument("--cut-every-nslices", type = int,
                metavar = "<samples>",
                action = "store", dest = "cut_every_nslices", default = None,
                help="how many slices should be groupped together at each cut")
        self.add_argument("--cut-until-nslices", type = int,
                metavar = "<slices>",
                action = "store", dest = "cut_until_nslices", default = None,
                help="how many extra slices should be added at the end of"
                " each slice")
        self.add_argument("--create-first",
                action = "store_true", dest = "create_first", default = False,
                help="always include first slice")

# some utilities

def samples2seconds(n_frames, samplerate):
    return "%f\t" % (n_frames / float(samplerate))

def samples2milliseconds(n_frames, samplerate):
    return "%f\t" % (1000. * n_frames / float(samplerate))

def samples2samples(n_frames, _samplerate):
    return "%d\t" % n_frames

def timefunc(mode):
    if mode is None or mode == 'seconds' or mode == 's':
        return samples2seconds
    elif mode == 'ms' or mode == 'milliseconds':
        return samples2milliseconds
    elif mode == 'samples':
        return samples2samples
    else:
        raise ValueError("invalid time format '%s'" % mode)

# definition of processing classes

class default_process(object):
    def __init__(self, args):
        if 'time_format' in args:
            self.time2string = timefunc(args.time_format)
        if args.verbose > 2 and hasattr(self, 'options'):
            name = type(self).__name__.split('_')[1]
            optstr = ' '.join(['running', name, 'with options',
                repr(self.options), '\n'])
            sys.stderr.write(optstr)
    def flush(self, frames_read, samplerate):
        # optionally called at the end of process
        pass

    def parse_options(self, args, valid_opts):
        # get any valid options found in a dictionnary of arguments
        options = {k: v for k, v in vars(args).items() if k in valid_opts}
        self.options = options

    def remap_pvoc_options(self, options):
        # FIXME: we need to remap buf_size to win_s, hop_size to hop_s
        # adjust python/ext/py-phasevoc.c to understand buf_size/hop_size
        if 'buf_size' in options:
            options['win_s'] = options['buf_size']
            del options['buf_size']
        if 'hop_size' in options:
            options['hop_s'] = options['hop_size']
            del options['hop_size']
        self.options = options

class process_onset(default_process):
    valid_opts = ['method', 'hop_size', 'buf_size', 'samplerate']
    def __init__(self, args):
        self.parse_options(args, self.valid_opts)
        self.onset = aubio.onset(**self.options)
        if args.threshold is not None:
            self.onset.set_threshold(args.threshold)
        if args.minioi:
            if args.minioi.endswith('ms'):
                self.onset.set_minioi_ms(float(args.minioi[:-2]))
            elif args.minioi.endswith('s'):
                self.onset.set_minioi_s(float(args.minioi[:-1]))
            else:
                self.onset.set_minioi(int(args.minioi))
        if args.silence:
            self.onset.set_silence(args.silence)
        super(process_onset, self).__init__(args)
    def __call__(self, block):
        return self.onset(block)
    def repr_res(self, res, _frames_read, samplerate):
        if res[0] != 0:
            outstr = self.time2string(self.onset.get_last(), samplerate)
            sys.stdout.write(outstr + '\n')

class process_pitch(default_process):
    valid_opts = ['method', 'hop_size', 'buf_size', 'samplerate']
    def __init__(self, args):
        self.parse_options(args, self.valid_opts)
        self.pitch = aubio.pitch(**self.options)
        if args.pitch_unit is not None:
            self.pitch.set_unit(args.pitch_unit)
        if args.threshold is not None:
            self.pitch.set_tolerance(args.threshold)
        if args.silence is not None:
            self.pitch.set_silence(args.silence)
        super(process_pitch, self).__init__(args)
    def __call__(self, block):
        return self.pitch(block)
    def repr_res(self, res, frames_read, samplerate):
        fmt_out = self.time2string(frames_read, samplerate)
        sys.stdout.write(fmt_out + "%.6f\n" % res[0])

class process_beat(default_process):
    valid_opts = ['method', 'hop_size', 'buf_size', 'samplerate']
    def __init__(self, args):
        self.parse_options(args, self.valid_opts)
        self.tempo = aubio.tempo(**self.options)
        super(process_beat, self).__init__(args)
    def __call__(self, block):
        return self.tempo(block)
    def repr_res(self, res, _frames_read, samplerate):
        if res[0] != 0:
            outstr = self.time2string(self.tempo.get_last(), samplerate)
            sys.stdout.write(outstr + '\n')

class process_tempo(process_beat):
    def __init__(self, args):
        super(process_tempo, self).__init__(args)
        self.beat_locations = []
    def repr_res(self, res, _frames_read, samplerate):
        if res[0] != 0:
            self.beat_locations.append(self.tempo.get_last_s())
    def flush(self, frames_read, samplerate):
        import numpy as np
        if len(self.beat_locations) < 2:
            outstr = "unknown bpm"
        else:
            bpms = 60. / np.diff(self.beat_locations)
            median_bpm = np.mean(bpms)
            if len(self.beat_locations) < 10:
                outstr = "%.2f bpm (uncertain)" % median_bpm
            else:
                outstr = "%.2f bpm" % median_bpm
        sys.stdout.write(outstr + '\n')

class process_notes(default_process):
    valid_opts = ['method', 'hop_size', 'buf_size', 'samplerate']
    def __init__(self, args):
        self.parse_options(args, self.valid_opts)
        self.notes = aubio.notes(**self.options)
        if args.silence is not None:
            self.notes.set_silence(args.silence)
        if args.release_drop is not None:
            self.notes.set_release_drop(args.release_drop)
        super(process_notes, self).__init__(args)
    def __call__(self, block):
        return self.notes(block)
    def repr_res(self, res, frames_read, samplerate):
        if res[2] != 0:  # note off
            fmt_out = self.time2string(frames_read, samplerate)
            sys.stdout.write(fmt_out + '\n')
        if res[0] != 0:  # note on
            lastmidi = res[0]
            fmt_out = "%f\t" % lastmidi
            fmt_out += self.time2string(frames_read, samplerate)
            sys.stdout.write(fmt_out)  # + '\t')
    def flush(self, frames_read, samplerate):
        eof = self.time2string(frames_read, samplerate)
        sys.stdout.write(eof + '\n')

class process_mfcc(default_process):
    def __init__(self, args):
        valid_opts1 = ['hop_size', 'buf_size']
        self.parse_options(args, valid_opts1)
        self.remap_pvoc_options(self.options)
        self.pv = aubio.pvoc(**self.options)

        valid_opts2 = ['buf_size', 'n_filters', 'n_coeffs', 'samplerate']
        self.parse_options(args, valid_opts2)
        self.mfcc = aubio.mfcc(**self.options)

        # remember all options
        self.parse_options(args, list(set(valid_opts1 + valid_opts2)))

        super(process_mfcc, self).__init__(args)

    def __call__(self, block):
        fftgrain = self.pv(block)
        return self.mfcc(fftgrain)
    def repr_res(self, res, frames_read, samplerate):
        fmt_out = self.time2string(frames_read, samplerate)
        fmt_out += ' '.join(["% 9.7f" % f for f in res.tolist()])
        sys.stdout.write(fmt_out + '\n')

class process_melbands(default_process):
    def __init__(self, args):
        self.args = args
        valid_opts = ['hop_size', 'buf_size']
        self.parse_options(args, valid_opts)
        self.remap_pvoc_options(self.options)
        self.pv = aubio.pvoc(**self.options)

        valid_opts = ['buf_size', 'n_filters']
        self.parse_options(args, valid_opts)
        self.remap_pvoc_options(self.options)
        self.filterbank = aubio.filterbank(**self.options)
        self.filterbank.set_mel_coeffs_slaney(args.samplerate)

        super(process_melbands, self).__init__(args)
    def __call__(self, block):
        fftgrain = self.pv(block)
        return self.filterbank(fftgrain)
    def repr_res(self, res, frames_read, samplerate):
        fmt_out = self.time2string(frames_read, samplerate)
        fmt_out += ' '.join(["% 9.7f" % f for f in res.tolist()])
        sys.stdout.write(fmt_out + '\n')

class process_quiet(default_process):
    def __init__(self, args):
        self.args = args
        valid_opts = ['hop_size', 'silence']
        self.parse_options(args, valid_opts)
        self.wassilence = 1

        if args.silence is not None:
            self.silence = args.silence
        super(process_quiet, self).__init__(args)

    def __call__(self, block):
        if aubio.silence_detection(block, self.silence) == 1:
            if self.wassilence != 1:
                self.wassilence = 1
                return 2   # newly found silence
            return 1       # silence again
        else:
            if self.wassilence != 0:
                self.wassilence = 0
                return -1  # newly found noise
            return 0       # noise again

    def repr_res(self, res, frames_read, samplerate):
        fmt_out = None
        if res == -1:
            fmt_out = "NOISY: "
        if res == 2:
            fmt_out = "QUIET: "
        if fmt_out is not None:
            fmt_out += self.time2string(frames_read, samplerate)
            sys.stdout.write(fmt_out + '\n')

class process_cut(process_onset):
    def __init__(self, args):
        super(process_cut, self).__init__(args)
        self.slices = []
        self.options = args

    def __call__(self, block):
        ret = super(process_cut, self).__call__(block)
        if ret:
            self.slices.append(self.onset.get_last())
        return ret

    def flush(self, frames_read, samplerate):
        _cut_slice(self.options, self.slices)
        duration = float(frames_read) / float(samplerate)
        base_info = '%(source_file)s' % \
                    {'source_file': self.options.source_uri}
        base_info += ' (total %(duration).2fs at %(samplerate)dHz)\n' % \
                     {'duration': duration, 'samplerate': samplerate}
        info = "created %d slices from " % len(self.slices)
        info += base_info
        sys.stderr.write(info)

def _cut_slice(options, timestamps):
    # cutting pass
    nstamps = len(timestamps)
    if nstamps > 0:
        # generate output files
        timestamps_end = None
        if options.cut_every_nslices:
            timestamps = timestamps[::options.cut_every_nslices]
            nstamps = len(timestamps)
        if options.cut_until_nslices and options.cut_until_nsamples:
            msg = "using cut_until_nslices, but cut_until_nsamples is set"
            warnings.warn(msg)
        if options.cut_until_nsamples:
            lag = options.cut_until_nsamples
            timestamps_end = [t + lag for t in timestamps[1:]]
            timestamps_end += [1e120]
        if options.cut_until_nslices:
            slice_lag = options.cut_until_nslices
            timestamps_end = [t for t in timestamps[1 + slice_lag:]]
            timestamps_end += [1e120] * (options.cut_until_nslices + 1)
        aubio.slice_source_at_stamps(options.source_uri,
                timestamps, timestamps_end = timestamps_end,
                output_dir = options.output_directory,
                samplerate = options.samplerate,
                create_first = options.create_first)

def main():
    parser = aubio_parser()
    if sys.version_info[0] != 3:
        # on py2, create a dummy ArgumentParser to workaround the
        # optional subcommand issue. See https://bugs.python.org/issue9253
        # This ensures that:
        #  - version string is shown when only '-V' is passed
        #  - help is printed if  '-V' is passed with any other argument
        #  - any other argument get forwarded to the real parser
        parser_root = argparse.ArgumentParser(add_help=False)
        parser_root.add_argument('-V', '--version', help="show version",
                action="store_true", dest="show_version")
        args, extras = parser_root.parse_known_args()
        if not args.show_version:  # no -V, forward to parser
            args = parser.parse_args(extras, namespace=args)
        elif len(extras) != 0:     # -V with other arguments, print help
            parser.print_help()
            sys.exit(1)
    else:  # in py3, we can simply use parser directly
        args = parser.parse_args()
    if 'show_version' in args and args.show_version:
        sys.stdout.write('aubio version ' + aubio.version + '\n')
        sys.exit(0)
    elif 'verbose' in args and args.verbose > 3:
        sys.stderr.write('aubio version ' + aubio.version + '\n')
    if 'command' not in args or args.command is None \
            or args.command in ['help']:
        # no command given, print help and return 1
        parser.print_help()
        if args.command and args.command in ['help']:
            sys.exit(0)
        else:
            sys.exit(1)
    elif not args.source_uri and not args.source_uri2:
        sys.stderr.write("Error: a source is required\n")
        parser.print_help()
        sys.exit(1)
    elif args.source_uri2 is not None:
        args.source_uri = args.source_uri2
    try:
        # open source_uri
        with aubio.source(args.source_uri, hop_size=args.hop_size,
                samplerate=args.samplerate) as a_source:
            # always update args.samplerate to native samplerate, in case
            # source was opened with args.samplerate=0
            args.samplerate = a_source.samplerate
            # create the processor for this subcommand
            processor = args.process(args)
            frames_read = 0
            while True:
                # read new block from source
                block, read = a_source()
                # execute processor on this block
                res = processor(block)
                # print results for this block
                if args.verbose > 0:
                    processor.repr_res(res, frames_read, a_source.samplerate)
                # increment total number of frames read
                frames_read += read
                # exit loop at end of file
                if read < a_source.hop_size:
                    break
            # flush the processor if needed
            processor.flush(frames_read, a_source.samplerate)
            if args.verbose > 1:
                fmt_string = "read {:.2f}s"
                fmt_string += " ({:d} samples in {:d} blocks of {:d})"
                fmt_string += " from {:s} at {:d}Hz\n"
                sys.stderr.write(fmt_string.format(
                        frames_read / float(a_source.samplerate),
                        frames_read,
                        frames_read // a_source.hop_size + 1,
                        a_source.hop_size,
                        a_source.uri,
                        a_source.samplerate))
    except KeyboardInterrupt:
        sys.exit(1)



#! /usr/bin/env python

from numpy.testing import TestCase

class aubiomodule_test_case(TestCase):

    def test_import(self):
        """ try importing aubio """
        import aubio

    def test_version(self):
        """ test aubio.version """
        import aubio
        self.assertEqual('0', aubio.version[0])

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

from numpy.testing import TestCase, assert_equal, assert_almost_equal
from aubio import notes, source
import numpy as np
from utils import list_all_sounds

list_of_sounds = list_all_sounds('sounds')

AUBIO_DEFAULT_NOTES_SILENCE = -70.
AUBIO_DEFAULT_NOTES_RELEASE_DROP = 10.
AUBIO_DEFAULT_NOTES_MINIOI_MS = 30.

class aubio_notes_default(TestCase):

    def test_members(self):
        o = notes()
        assert_equal ([o.buf_size, o.hop_size, o.method, o.samplerate],
            [1024,512,'default',44100])


class aubio_notes_params(TestCase):

    samplerate = 44100

    def setUp(self):
        self.o = notes(samplerate = self.samplerate)

    def test_get_minioi_ms(self):
        assert_equal (self.o.get_minioi_ms(), AUBIO_DEFAULT_NOTES_MINIOI_MS)

    def test_set_minioi_ms(self):
        val = 40.
        self.o.set_minioi_ms(val)
        assert_almost_equal (self.o.get_minioi_ms(), val)

    def test_get_silence(self):
        assert_equal (self.o.get_silence(), AUBIO_DEFAULT_NOTES_SILENCE)

    def test_set_silence(self):
        val = -50
        self.o.set_silence(val)
        assert_equal (self.o.get_silence(), val)

    def test_get_release_drop(self):
        assert_equal (self.o.get_release_drop(), AUBIO_DEFAULT_NOTES_RELEASE_DROP)

    def test_set_release_drop(self):
        val = 50
        self.o.set_release_drop(val)
        assert_equal (self.o.get_release_drop(), val)

    def test_set_release_drop_wrong(self):
        val = -10
        with self.assertRaises(ValueError):
            self.o.set_release_drop(val)

class aubio_notes_sinewave(TestCase):

    def analyze_file(self, filepath, samplerate=0):
        win_s = 512 # fft size
        hop_s = 256 # hop size

        s = source(filepath, samplerate, hop_s)
        samplerate = s.samplerate

        tolerance = 0.8

        notes_o = notes("default", win_s, hop_s, samplerate)
        total_frames = 0

        results = []
        while True:
            samples, read = s()
            new_note = notes_o(samples)
            if (new_note[0] != 0):
                note_str = ' '.join(["%.2f" % i for i in new_note])
                results.append( [total_frames, np.copy(new_note)] )
            total_frames += read
            if read < hop_s: break
        return results

    def test_sinewave(self):
        for filepath in list_of_sounds:
            if '44100Hz_44100f_sine441.wav' in filepath:
                results = self.analyze_file(filepath)
                assert_equal (len(results), 1)
                assert_equal (len(results[0]), 2)
                assert_equal (results[0][0], 1280)
                assert_equal (results[0][1], [69, 123, -1])

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

"""A brute force test using `sink` to create and write samples to a stereo
file, then `source` to check the correct content is read from the files."""

import os.path
import unittest
import aubio
import numpy as np
from numpy.testing import assert_equal
from utils import get_tmp_sink_path

class aubio_source_test_case(unittest.TestCase):

    def test_read_from_mono(self):
        out = get_tmp_sink_path()
        samplerate = 44100
        hop_size = 256
        blocks = 10
        channels = 1
        write_samples = np.ones([channels, hop_size], dtype=aubio.float_type)
        write_samples *= .5
        self.check_write_and_read(samplerate, channels, hop_size, blocks,
                write_samples)

    def test_read_from_stereo(self):
        out = get_tmp_sink_path()
        samplerate = 44100
        hop_size = 256
        blocks = 10
        channels = 1
        write_samples = np.ones([channels, hop_size], dtype=aubio.float_type)
        write_samples *= .5
        self.check_write_and_read(samplerate, channels, hop_size, blocks,
                write_samples)

    def test_read_from_half_stereo(self):
        samplerate = 16000
        channels = 2
        hop_size = 512
        blocks = 10
        write_samples = np.ones([channels, hop_size], dtype=aubio.float_type)
        write_samples *= .5
        write_samples[1, :] = 0
        self.check_write_and_read(samplerate, channels, hop_size, blocks,
                write_samples)

    def test_read_from_cancelling_channels(self):
        samplerate = 16000
        channels = 2
        hop_size = 512
        blocks = 10
        write_samples = np.ones([channels, hop_size], dtype=aubio.float_type)
        write_samples *= .5
        write_samples[1] *= -1
        self.check_write_and_read(samplerate, channels, hop_size, blocks,
                write_samples)

    def test_read_from_strange_three_channels(self):
        samplerate = 8000
        channels = 3
        hop_size = 123
        blocks = 10
        write_samples = np.ones([channels, hop_size], dtype=aubio.float_type)
        write_samples *= .5
        write_samples[1, :] = 0
        self.check_write_and_read(samplerate, channels, hop_size, blocks,
                write_samples)

    def check_write_and_read(self, samplerate, channels,
            hop_size, blocks, write_samples):
        expected_mono = np.sum(write_samples, axis=0)/write_samples.shape[0]
        out = get_tmp_sink_path()
        snk = aubio.sink(out, samplerate, channels=channels)
        for i in range(blocks):
            snk.do_multi(write_samples, hop_size)
        # close the sink before reading from it
        snk.close()

        src = aubio.source(out, samplerate, hop_size)
        for i in range(blocks):
            read_samples, read = src.do_multi()
            assert_equal (read_samples, write_samples)
            assert_equal (read, hop_size)

        src.seek(0)
        for i in range(blocks):
            read_samples, read = src()
            assert_equal (read, hop_size)
            assert_equal (read_samples, expected_mono)

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aubio import note2midi, freq2note, note2freq, float_type
from numpy.testing import TestCase
from _tools import parametrize, assert_raises, skipTest

list_of_known_notes = (
        ( 'C-1', 0 ),
        ( 'C#-1', 1 ),
        ( 'd2', 38 ),
        ( 'C3', 48 ),
        ( 'B3', 59 ),
        ( 'B#3', 60 ),
        ( 'C♯4', 61 ),
        ( 'A4', 69 ),
        ( 'A#4', 70 ),
        ( 'A♯4', 70 ),
        ( 'A\u266f4', 70 ),
        ( 'Bb4', 70 ),
        ( 'B♭4', 70 ),
        ( 'B\u266d4', 70 ),
        ( 'G8', 115 ),
        ( 'G♯8', 116 ),
        ( 'G9', 127 ),
        ( 'A♮2', 45 ),
        )

list_of_known_notes_with_unicode_issues = (
        ('C𝄪4', 62 ),
        ('E𝄫4', 62 ),
        )

list_of_unknown_notes = (
        ( 'G\udd2a2' ),
        ( 'B\ufffd2' ),
        ( 'B\u266e\u266e2' ),
        ( 'B\u266f\u266d3' ),
        ( 'B33' ),
        ( 'C.3' ),
        ( 'A' ),
        ( '2' ),
        )

class Test_note2midi_good_values(object):

    @parametrize('note, midi', list_of_known_notes)
    def test_note2midi_known_values(self, note, midi):
        " known values are correctly converted "
        assert note2midi(note) == midi

    @parametrize('note, midi', list_of_known_notes_with_unicode_issues)
    def test_note2midi_known_values_with_unicode_issues(self, note, midi):
        " difficult values are correctly converted unless expected failure "
        try:
            assert note2midi(note) == midi
        except UnicodeEncodeError as e:
            # platforms with decoding failures include:
            # - osx: python <= 2.7.10
            # - win: python <= 2.7.12
            import sys
            strmsg = "len(u'\\U0001D12A') != 1, expected decoding failure"
            strmsg += " | upgrade to Python 3 to fix"
            strmsg += " | {:s} | {:s} {:s}"
            if len('\U0001D12A') != 1 and sys.version[0] == '2':
                skipTest(strmsg.format(repr(e), sys.platform, sys.version))
            else:
                raise

class note2midi_wrong_values(TestCase):

    def test_note2midi_missing_octave(self):
        " fails when passed only one character"
        self.assertRaises(ValueError, note2midi, 'C')

    def test_note2midi_wrong_modifier(self):
        " fails when passed a note with an invalid modifier "
        self.assertRaises(ValueError, note2midi, 'C.1')

    def test_note2midi_another_wrong_modifier_again(self):
        " fails when passed a note with a invalid note name "
        self.assertRaises(ValueError, note2midi, 'CB-3')

    def test_note2midi_wrong_octave(self):
        " fails when passed a wrong octave number "
        self.assertRaises(ValueError, note2midi, 'CBc')

    def test_note2midi_out_of_range(self):
        " fails when passed a note out of range"
        self.assertRaises(ValueError, note2midi, 'A9')

    def test_note2midi_wrong_note_name(self):
        " fails when passed a note with a wrong name"
        self.assertRaises(ValueError, note2midi, 'W9')

    def test_note2midi_low_octave(self):
        " fails when passed a note with a too low octave"
        self.assertRaises(ValueError, note2midi, 'C-9')

    def test_note2midi_wrong_data_type(self):
        " fails when passed a non-string value "
        self.assertRaises(TypeError, note2midi, 123)

    def test_note2midi_wrong_data_too_long(self):
        " fails when passed a note with a note name longer than expected"
        self.assertRaises(ValueError, note2midi, 'CB+-3')

class Test_note2midi_unknown_values(object):

    @parametrize('note', list_of_unknown_notes)
    def test_note2midi_unknown_values(self, note):
        " unknown values throw out an error "
        assert_raises(ValueError, note2midi, note)

class freq2note_simple_test(TestCase):

    def test_freq2note_above(self):
        " make sure freq2note(441) == A4 "
        self.assertEqual("A4", freq2note(441))

    def test_freq2note_under(self):
        " make sure freq2note(439) == A4 "
        self.assertEqual("A4", freq2note(439))

class note2freq_simple_test(TestCase):

    def test_note2freq(self):
        " make sure note2freq('A3') == 220"
        self.assertEqual(220, note2freq("A3"))

    def test_note2freq_under(self):
        " make sure note2freq(A4) == 440"
        if float_type == 'float32':
            self.assertEqual(440, note2freq("A4"))
        else:
            self.assertLess(abs(note2freq("A4")-440), 1.e-12)

if __name__ == '__main__':
    from _tools import run_module_suite
    run_module_suite()



#! /usr/bin/env python

import numpy as np
from aubio import pitch
import pylab as plt

buf_size = 2048 * 1
hop_size = buf_size // 4

samplerate = 44100
minfreq = 40
maxfreq = 6000

def sinewave(freq, duration, samplerate = samplerate):
    """ generate a sinewave """
    length = hop_size
    while length < duration * samplerate:
        length += hop_size
    return np.sin( 2. * np.pi * np.arange(length) * freq / samplerate ).astype("float32")

def get_stats_for_pitch_method(method, freqs, samplerate = samplerate):
    """ for a given pitch method and a list of frequency, generate a sinewave
    and get mean deviation """
    means = np.zeros(len(freqs))
    medians = np.zeros(len(freqs))
    for freq, fn in zip(freqs, range(len(freqs))):
        s = sinewave(freq, .50).reshape(-1, hop_size)
        #s = (sinewave(freq, .50) + .0*sinewave(freq/2., .50)).reshape(-1, hop_size)
        p = pitch(method, buf_size, hop_size, samplerate = samplerate)
        candidates = np.zeros(len(s))
        #samples = np.zeros(buf_size)
        for frame, i in zip(s, range(len(s))):
            candidates[i] = p(frame)[0]
        # skip first few candidates
        candidates = candidates[4:]
        means[fn] = np.mean(candidates[candidates != 0] - freq)
        medians[fn] = np.median(candidates[candidates != 0] - freq)
        print (freq, means[fn], medians[fn])
    return means, medians

if __name__ == '__main__':
    freqs = np.arange(minfreq, maxfreq, 1.)
    modes = ["yin", "yinfft"]
    for mode in modes:
        means, medians = get_stats_for_pitch_method(mode, freqs)
        plt.figure()
        plt.plot(freqs, means, 'g-')
        plt.plot(freqs, medians, 'r--')
        #plt.savefig(mode + '_deviations_test.png', dpi=300)
        plt.show()



#! /usr/bin/env python

from numpy.testing import TestCase, assert_equal
from numpy import sin, arange, mean, median, isnan, pi
from aubio import fvec, pitch, freqtomidi, float_type

class aubio_pitch_Good_Values(TestCase):

    def skip_test_new_default(self):
        " creating a pitch object without parameters "
        p = pitch()
        assert_equal ( [p.method, p.buf_size, p.hop_size, p.samplerate],
            ['default', 1024, 512, 44100])

    def test_run_on_silence(self):
        " creating a pitch object with parameters "
        p = pitch('default', 2048, 512, 32000)
        assert_equal ( [p.method, p.buf_size, p.hop_size, p.samplerate],
            ['default', 2048, 512, 32000])

    def test_run_on_zeros(self):
        " running on silence gives 0 "
        p = pitch('default', 2048, 512, 32000)
        f = fvec (512)
        for _ in range(10): assert_equal (p(f), 0.)

    def test_run_on_ones(self):
        " running on ones gives 0 "
        p = pitch('default', 2048, 512, 32000)
        f = fvec (512)
        f[:] = 1
        for _ in range(10): assert_equal (p(f), 0.)

class aubio_pitch_Sinusoid(TestCase):

    def run_pitch_on_sinusoid(self, method, buf_size, hop_size, samplerate, freq):
        # create pitch object
        p = pitch(method, buf_size, hop_size, samplerate)
        # duration in seconds
        seconds = .3
        # duration in samples
        duration =  seconds * samplerate
        # increase to the next multiple of hop_size
        duration = duration - duration % hop_size + hop_size;
        # build sinusoid
        sinvec = self.build_sinusoid(duration, freq, samplerate)

        self.run_pitch(p, sinvec, freq)

    def build_sinusoid(self, length, freq, samplerate):
        return sin( 2. * pi * arange(length).astype(float_type) * freq / samplerate)

    def run_pitch(self, p, input_vec, freq):
        pitches, errors = [], []
        input_blocks = input_vec.reshape((-1, p.hop_size))
        for new_block in input_blocks:
            pitch = p(new_block)[0]
            pitches.append(pitch)
            errors.append(1. - freqtomidi(pitch) / freqtomidi(freq))
        assert_equal ( len(input_blocks), len(pitches) )
        assert_equal ( isnan(pitches), False )
        # cut the first candidates
        #cut = ( p.buf_size - p.hop_size ) / p.hop_size
        pitches = pitches[2:]
        errors = errors[2:]
        # check that the mean of all relative errors is less than 10%
        #assert abs (mean(errors) ) < 0.1, pitches
        assert abs (median(errors) ) < 0.06, "median of relative errors is bigger than 0.06 (%f)\n found %s\n errors %s" % (mean(errors), pitches, errors)
        #print 'len(pitches), cut:', len(pitches), cut
        #print 'median errors: ', median(errors), 'median pitches: ', median(pitches)

pitch_algorithms = [ "default", "yinfft", "yin", "yinfast", "schmitt", "mcomb", "fcomb" , "specacf" ]
pitch_algorithms = [ "default", "yinfft", "yin", "yinfast", "schmitt", "mcomb", "fcomb" ]

#freqs = [ 27.5, 55., 110., 220., 440., 880., 1760., 3520. ]
freqs = [             110., 220., 440., 880., 1760., 3520. ]
signal_modes = []
for freq in freqs:
    signal_modes += [
        ( 4096,  2048, 44100, freq ),
        ( 2048,  512, 44100, freq ),
        ( 2048, 1024, 44100, freq ),
        ( 2048, 1024, 32000, freq ),
        ]

freqs = [ ] #55., 110., 220., 440., 880., 1760., 3520. ]
for freq in freqs:
    signal_modes += [
        ( 2048, 1024, 22050, freq ),
        ( 1024,  256, 16000, freq ),
        ( 1024,  256, 8000,  freq ),
        ( 1024, 512+256, 8000, freq ),
        ]

"""
signal_modes = [
        ( 4096,  512, 44100, 2.*882. ),
        ( 4096,  512, 44100, 882. ),
        ( 4096,  512, 44100, 440. ),
        ( 2048,  512, 44100, 440. ),
        ( 2048, 1024, 44100, 440. ),
        ( 2048, 1024, 44100, 440. ),
        ( 2048, 1024, 32000, 440. ),
        ( 2048, 1024, 22050, 440. ),
        ( 1024,  256, 16000, 440. ),
        ( 1024,  256, 8000,  440. ),
        ( 1024, 512+256, 8000, 440. ),
        ]
"""

def create_test (algo, mode):
    def do_test_pitch(self):
        self.run_pitch_on_sinusoid(algo, mode[0], mode[1], mode[2], mode[3])
    return do_test_pitch

for algo in pitch_algorithms:
    for mode in signal_modes:
        _test_method = create_test (algo, mode)
        _test_method.__name__ = 'test_pitch_%s_%d_%d_%dHz_sin_%.0f' % ( algo,
                mode[0], mode[1], mode[2], mode[3] )
        setattr (aubio_pitch_Sinusoid, _test_method.__name__, _test_method)

if __name__ == '__main__':
    from unittest import main
    main()



#! /usr/bin/env python

from numpy.testing import TestCase
from aubio import fvec, source, sink
from utils import list_all_sounds, get_tmp_sink_path, del_tmp_sink_path
from utils import parse_file_samplerate
from _tools import parametrize, skipTest, assert_raises, assert_warns

list_of_sounds = list_all_sounds('sounds')
samplerates = [0, 44100, 8000, 32000]
hop_sizes = [512, 1024, 64]

path = None

many_files = 300 # 256 opened files is too much

all_params = []
for soundfile in list_of_sounds:
    for hop_size in hop_sizes:
        for samplerate in samplerates:
            all_params.append((hop_size, samplerate, soundfile))

class Test_aubio_sink(object):

    def test_wrong_filename(self):
        with assert_raises(RuntimeError):
            sink('')

    def test_wrong_samplerate(self):
        with assert_raises(RuntimeError):
            sink(get_tmp_sink_path(), -1)

    def test_wrong_samplerate_too_large(self):
        with assert_raises(RuntimeError):
            sink(get_tmp_sink_path(), 1536001, 2)

    def test_wrong_channels(self):
        with assert_raises(RuntimeError):
            sink(get_tmp_sink_path(), 44100, -1)

    def test_wrong_channels_too_large(self):
        with assert_raises(RuntimeError):
            sink(get_tmp_sink_path(), 44100, 202020)

    def test_many_sinks(self):
        from tempfile import mkdtemp
        import os.path
        import shutil
        tmpdir = mkdtemp()
        sink_list = []
        for i in range(many_files):
            path = os.path.join(tmpdir, 'f-' + str(i) + '.wav')
            g = sink(path, 0)
            sink_list.append(g)
            write = 32
            for _ in range(200):
                vec = fvec(write)
                g(vec, write)
            g.close()
        shutil.rmtree(tmpdir)

    @parametrize('hop_size, samplerate, path', all_params)
    def test_read_and_write(self, hop_size, samplerate, path):
        orig_samplerate = parse_file_samplerate(soundfile)
        try:
            if orig_samplerate is not None and orig_samplerate < samplerate:
                # upsampling should emit a warning
                with assert_warns(UserWarning):
                    f = source(soundfile, samplerate, hop_size)
            else:
                f = source(soundfile, samplerate, hop_size)
        except RuntimeError as e:
            err_msg = '{:s} (hop_s = {:d}, samplerate = {:d})'
            skipTest(err_msg.format(str(e), hop_size, samplerate))
        if samplerate == 0: samplerate = f.samplerate
        sink_path = get_tmp_sink_path()
        g = sink(sink_path, samplerate)
        total_frames = 0
        while True:
            vec, read = f()
            g(vec, read)
            total_frames += read
            if read < f.hop_size: break
        del_tmp_sink_path(sink_path)

    @parametrize('hop_size, samplerate, path', all_params)
    def test_read_and_write_multi(self, hop_size, samplerate, path):
        orig_samplerate = parse_file_samplerate(soundfile)
        try:
            if orig_samplerate is not None and orig_samplerate < samplerate:
                # upsampling should emit a warning
                with assert_warns(UserWarning):
                    f = source(soundfile, samplerate, hop_size)
            else:
                f = source(soundfile, samplerate, hop_size)
        except RuntimeError as e:
            err_msg = '{:s} (hop_s = {:d}, samplerate = {:d})'
            skipTest(err_msg.format(str(e), hop_size, samplerate))
        if samplerate == 0: samplerate = f.samplerate
        sink_path = get_tmp_sink_path()
        g = sink(sink_path, samplerate, channels = f.channels)
        total_frames = 0
        while True:
            vec, read = f.do_multi()
            g.do_multi(vec, read)
            total_frames += read
            if read < f.hop_size: break
        del_tmp_sink_path(sink_path)

    def test_close_file(self):
        samplerate = 44100
        sink_path = get_tmp_sink_path()
        g = sink(sink_path, samplerate)
        g.close()
        del_tmp_sink_path(sink_path)

    def test_close_file_twice(self):
        samplerate = 44100
        sink_path = get_tmp_sink_path()
        g = sink(sink_path, samplerate)
        g.close()
        g.close()
        del_tmp_sink_path(sink_path)

    def test_read_with(self):
        samplerate = 44100
        sink_path = get_tmp_sink_path()
        vec = fvec(128)
        with sink(sink_path, samplerate) as g:
            for _ in range(10):
                g(vec, 128)

if __name__ == '__main__':
    from _tools import run_module_suite
    run_module_suite()



#! /usr/bin/env python

def record_sink(sink_path):
    """Record an audio file using pysoundcard."""

    from aubio import sink
    from pysoundcard import Stream

    hop_size = 256
    duration = 5 # in seconds
    s = Stream(blocksize = hop_size, channels = 1)
    g = sink(sink_path, samplerate = int(s.samplerate))

    s.start()
    total_frames = 0
    try:
        while total_frames < duration * s.samplerate:
            vec = s.read(hop_size)
            # mix down to mono
            mono_vec = vec.sum(-1) / float(s.channels[0])
            g(mono_vec, hop_size)
            total_frames += hop_size
    except KeyboardInterrupt:
        duration = total_frames / float(s.samplerate)
        print("stopped after %.2f seconds" % duration)
    s.stop()

if __name__ == '__main__':
    import sys
    record_sink(sys.argv[1])



#! /usr/bin/env python

# Simple demo to extract notes from a sound file, and store them in a midi file
# using mido.
#
# Install mido: `pip instal mido`
#
# Documentation: https://mido.readthedocs.io/

import sys
from aubio import source, notes
from mido import Message, MetaMessage, MidiFile, MidiTrack, second2tick, bpm2tempo

if len(sys.argv) < 3:
    print("Usage: %s <filename> <output> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]
midioutput = sys.argv[2]

downsample = 1
samplerate = 44100 // downsample
if len( sys.argv ) > 3: samplerate = int(sys.argv[3])

win_s = 512 // downsample # fft size
hop_s = 256 // downsample # hop size

s = source(filename, samplerate, hop_s)
samplerate = s.samplerate

tolerance = 0.8

notes_o = notes("default", win_s, hop_s, samplerate)

print("%8s" % "time","[ start","vel","last ]")

# create a midi file
mid = MidiFile()
track = MidiTrack()
mid.tracks.append(track)

ticks_per_beat = mid.ticks_per_beat # default: 480
bpm = 120 # default midi tempo

tempo = bpm2tempo(bpm)
track.append(MetaMessage('set_tempo', tempo=tempo))
track.append(MetaMessage('time_signature', numerator=4, denominator=4))

def frames2tick(frames, samplerate=samplerate):
    sec = frames / float(samplerate)
    return int(second2tick(sec, ticks_per_beat, tempo))

last_time = 0

# total number of frames read
total_frames = 0
while True:
    samples, read = s()
    new_note = notes_o(samples)
    if (new_note[0] != 0):
        note_str = ' '.join(["%.2f" % i for i in new_note])
        print("%.6f" % (total_frames/float(samplerate)), new_note)
        delta = frames2tick(total_frames) - last_time
        if new_note[2] > 0:
            track.append(Message('note_off', note=int(new_note[2]),
                velocity=127, time=delta)
                )
        track.append(Message('note_on',
            note=int(new_note[0]),
            velocity=int(new_note[1]),
            time=delta)
            )
        last_time = frames2tick(total_frames)
    total_frames += read
    if read < hop_s: break

mid.save(midioutput)



#! /usr/bin/env python

import sys
from aubio import source, notes

if len(sys.argv) < 2:
    print("Usage: %s <filename> [samplerate]" % sys.argv[0])
    sys.exit(1)

filename = sys.argv[1]

downsample = 1
samplerate = 44100 // downsample
if len( sys.argv ) > 2: samplerate = int(sys.argv[2])

win_s = 512 // downsample # fft size
hop_s = 256 // downsample # hop size

s = source(filename, samplerate, hop_s)
samplerate = s.samplerate

tolerance = 0.8

notes_o = notes("default", win_s, hop_s, samplerate)

print("%8s" % "time","[ start","vel","last ]")

# total number of frames read
total_frames = 0
while True:
    samples, read = s()
    new_note = notes_o(samples)
    if (new_note[0] != 0):
        note_str = ' '.join(["%.2f" % i for i in new_note])
        print("%.6f" % (total_frames/float(samplerate)), new_note)
    total_frames += read
    if read < hop_s: break
